import logging
import os

from PySide6.QtCore import QTimer
from PySide6.QtGui import QIcon, QPixmap, QAction
from PySide6.QtWidgets import QMainWindow, QProgressDialog, QToolButton, QMenu
from WidgetCollection.Dialogs import AboutDialog
from rich.logging import RichHandler

from SacherECLControl.controller.LaserDeviceControl import MPLaserDeviceControl
from SacherECLControl.model.LaserControlModel import LaserControlModel
from SacherECLControl.view.Ui_LaserControlWindow import Ui_LaserControlWindow
from SacherECLControl.view.WidgetLaserInformation import WidgetLaserInformation


from SacherECLControl import __version__, __author__, __license__, __description__, __url__
class LaserControlView(QMainWindow):

    def __init__(self, model: LaserControlModel, controller: MPLaserDeviceControl):
        super().__init__()

        self.logger = controller.logger

        self._ui = Ui_LaserControlWindow()
        self._ui.setupUi(self)

        self.setWindowTitle(f'Sacher External Cavity Laser Control - {__version__}')
        self.setWindowIcon(QIcon(':/icons/icons/sacherelccontrol_icon.png'))

        self.about_dialog = AboutDialog(
            "ADScope Control",
            __description__,
            __version__,
            f"2024 {__author__}",
            __url__,
            f"This project is open source and contributions are highly welcome.<br>"
            f"<br>The project is licensed under <br>{__license__}.",
            QPixmap(':/icons/icons/sacherelccontrol_icon.png')
        )

        self._init_menu_bar()

        self._enable_ui(enabled=False)

        self.laser_moving_progress_dialog = QProgressDialog(self)
        self.laser_moving_progress_dialog.cancel()
        self.timer = QTimer()
        self.timer.timeout.connect(self.handle_timer)

        self.model: LaserControlModel = model
        self.controller = controller

        self.model.laser_config.velocity.view.add_new_view(self._ui.sb_velocity)
        self.model.laser_config.acceleration.view.add_new_view(self._ui.sb_acc)
        self.model.laser_config.deceleration.view.add_new_view(self._ui.sb_dec)
        self.model.laser_config.wl_sweep_start.view.add_new_view(self._ui.sb_sweep_start)
        self.model.laser_config.wl_sweep_stop.view.add_new_view(self._ui.sb_sweep_stop)

        # Laser Information
        self.laser_information = WidgetLaserInformation()
        self._ui.grd_system_info.addWidget(self.laser_information, 0, 0, 1, 1)

        self._connect_controls()
        self._connect_signals()

    # def _on_uncertainty_changed(self, uncertainty):
    #    self.uncertainty_dist_plot.uncertainty = uncertainty

    def _init_menu_bar(self):
        # Add menu bar
        self.file_menu = QMenu('&Files', self)

        self.act_connect = QAction('Connect', self)
        self.file_menu.addAction(self.act_connect)

        self.file_menu.addSeparator()

        self.file_menu.addSeparator()

        self.act_about = QAction('About', self)
        self.act_about.triggered.connect(self.about_dialog.exec)
        self.file_menu.addAction(self.act_about)

        self.act_exit = QAction('Exit', self)
        self.act_exit.setShortcut('Ctrl+Q')
        self.file_menu.addAction(self.act_exit)

        self._ui.menu_file.setMenu(self.file_menu)
        self._ui.menu_file.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)

    def connect_capturing_device(self, device):
        pass

    def _connect_controls(self):
        # USB Connection List
        self._ui.cb_port_selection.addItems(self.model.laser_config.available_ports.get())
        self._ui.cb_port_selection.currentTextChanged.connect(self._on_port_changed)

        # Buttons
        self._ui.btn_connect.clicked.connect(self._on_btn_connect_clicked)
        self._ui.btn_move_to_wavelength.clicked.connect(self._on_btn_move_to_wavelength_clicked)

        # Spinboxes
        self.model.port = self._ui.cb_port_selection.currentText()

        # Sweep start
        self._ui.btn_start_sweep.clicked.connect(self._on_btn_start_sweep_clicked)

        # Connect the Spinboxes for the sweep settings. A change in values sets the new sweep settings
        self._ui.sb_sweep_start.editingFinished.connect(self._on_sb_sweep_start_changed)
        self._ui.sb_sweep_stop.editingFinished.connect(self._on_sb_sweep_stop_changed)

        #self._ui.btn_save_velocity_params.clicked.connect(self._on_btn_save_velocity_params_clicked)
        self._ui.sb_velocity.editingFinished.connect(self._on_sb_velocity_editing_finished)
        self._ui.sb_acc.editingFinished.connect(self._on_sb_acc_editing_finished)
        self._ui.sb_dec.editingFinished.connect(self._on_sb_dec_editing_finished)

    def _connect_signals(self):

        # Triggerd if the laser connection status changes
        self.model.signals.connected_changed.connect(self._on_laser_connected_changed)
        self.model.signals.laser_is_moving_changed.connect(self._on_laser_is_moving_changed)

        # Capturing device signals
        self.model.signals.capturing_device_connected_changed.connect(self._on_capture_device_connected_changed)

        # self.uncertainty_model.signals.uncertainty_changed.connect(self._on_uncertainty_changed)

        # Triggerd if the laser port changes
        self.model.signals.current_wavelength_changed.connect(self._on_wavelength_changed)
        self.model.signals.min_laser_wavelength_changed.connect(self._on_min_wavelength_changed)
        self.model.signals.max_laser_wavelength_changed.connect(self._on_max_wavelength_changed)

        self.model.signals.velocity_changed.connect(self._on_velocity_changed)
        self.model.signals.acceleration_changed.connect(self._on_acceleration_changed)
        self.model.signals.deceleration_changed.connect(self._on_deceleration_changed)

        # Sweep Settings
        self.model.signals.sweep_start_wavelength_changed.connect(self._on_sweep_start_changed)
        self.model.signals.sweep_stop_wavelength_changed.connect(self._on_sweep_stop_changed)

        # ==================================================================================================================
        # Slots that are triggerd if the laser connection changes
        # ==================================================================================================================

    def _enable_ui(self, enabled=True):
        self._ui.group_motor_profile.setEnabled(enabled)
        self._ui.group_wavelength_settings.setEnabled(enabled)
        self._ui.group_sweep_settings.setEnabled(enabled)

    def _on_laser_connected_changed(self, connected):
        if connected:
            self._ui.btn_connect.setText("Disconnect")
            self.laser_information.set_connection_state(True, f"Connected ({self.model.port})")
        else:
            self._ui.btn_connect.setText("Connect")
            self.laser_information.set_connection_state(False)
        self._enable_ui(connected)

    def _on_laser_is_moving_changed(self, is_moving, to_wl):
        self._ui.btn_move_to_wavelength.setEnabled(not is_moving)
        self._ui.btn_start_sweep.setEnabled(not is_moving)
        self.laser_information.set_movement_state(is_moving, "Moving" if is_moving else "Stopped")
        if is_moving:
            self.logger.info(f"Moving from {self.model.current_wavelength} nm to  {to_wl} nm")
            self._display_estimated_progress(
                self.model.current_wavelength,
                self.model.laser_moving_to_wavelength)
        else:
            self._exit_display_estimated_progress()

    # ==================================================================================================================
    # Slots that are triggerd if the laser settings change7
    # ==================================================================================================================
    #def _on_connected_changed(self, connected):
    #    self.laser_information.set_connection_state(connected)
    #

    # ==================================================================================================================
    # Slots that are triggered if the capturing device changes
    # ==================================================================================================================
    def _on_capture_device_connected_changed(self, connected):
        if connected:
            self.laser_information.set_capt_dev_state(connected, self.model.capturing_device.model.device_information.device_name)
            self.model.capturing_device.model.device_information.signals.device_name_changed.connect(
                self._on_capture_device_name_changed)

    def _on_capture_device_name_changed(self):
        self.laser_information.set_capt_dev_state(self.model.capturing_device_connected,
                                                  f"Connected: {self.model.capturing_device.model.device_information.device_name}")

    def _on_wavelength_changed(self, wavelength):
        self.logger.debug(f"Wavelength changed to: {wavelength}. "
                    f"Range ({self.model.min_laser_wavelength}-{self.model.max_laser_wavelength})nm")

        # self._ui.lbl_min_wavelength.setText(str(self.model.min_laser_wavelength))
        # self._ui.lbl_max_wavelength.setText(str(self.model.max_laser_wavelength))

        # self._ui.slider_wavelength.setMinimum(self.model.min_laser_wavelength)
        # self._ui.slider_wavelength.setMaximum(self.model.max_laser_wavelength)

        # self._ui.sb_set_wavelength.setMinimum(self.model.min_laser_wavelength)
        # self._ui.sb_set_wavelength.setMaximum(self.model.max_laser_wavelength)

        # Set the minimum and maximum values for the sweep start and stop
        self._ui.sb_set_wavelength.setValue(wavelength)
        self._ui.lcd_wavelength.display(wavelength)
        self._ui.slider_wavelength.setValue(wavelength)

    def _on_min_wavelength_changed(self, min_wavelength):
        self._ui.lbl_min_wavelength.setText(str(self.model.min_laser_wavelength))
        self._ui.slider_wavelength.setMinimum(self.model.min_laser_wavelength)
        self._ui.sb_set_wavelength.setMinimum(self.model.min_laser_wavelength)

    def _on_max_wavelength_changed(self, min_wavelength):
        self._ui.lbl_max_wavelength.setText(str(self.model.max_laser_wavelength))
        self._ui.slider_wavelength.setMaximum(self.model.max_laser_wavelength)
        self._ui.sb_set_wavelength.setMaximum(self.model.max_laser_wavelength)

    def _on_velocity_changed(self, velocity: float):
        self.logger.debug(f"Velocity changed to: {velocity} "
                          f"[{self.model.min_velocity}, {self.model.max_velocity}]")

        self._ui.sb_velocity.setMinimum(self.model.min_velocity)
        self._ui.sb_velocity.setMaximum(self.model.max_velocity)
        self._ui.sb_velocity.setValue(velocity)

    def _on_deceleration_changed(self, deceleration: float):
        self.logger.debug(f"Deceleration changed to: {deceleration} "
                          f"[{self.model.min_deceleration}, {self.model.max_deceleration}]")

        self._ui.sb_dec.setMinimum(self.model.min_deceleration)
        self._ui.sb_dec.setMaximum(self.model.max_deceleration)
        self._ui.sb_dec.setValue(deceleration)

    def _on_acceleration_changed(self, acceleration: float):
        self.logger.debug(f"Acceleration changed to: {acceleration} "
                          f"[{self.model.min_acceleration}, {self.model.max_acceleration}]")

        self._ui.sb_acc.setMinimum(self.model.min_acceleration)
        self._ui.sb_acc.setMaximum(self.model.max_acceleration)
        self._ui.sb_acc.setValue(acceleration)

    def _on_sweep_start_changed(self, sweep_start):
        self.logger.debug(f"View: Sweep start changed. "
                          f"Min {self.model.min_laser_wavelength}, Max {self.model.max_laser_wavelength}")
        self._ui.sb_sweep_start.setRange(self.model.min_laser_wavelength, self.model.max_laser_wavelength)
        self._ui.sb_sweep_start.setValue(sweep_start)

    def _on_sweep_stop_changed(self, sweep_stop):
        self._ui.sb_sweep_stop.setRange(self.model.min_laser_wavelength, self.model.max_laser_wavelength)
        self._ui.sb_sweep_stop.setValue(sweep_stop)

        # ==================================================================================================================
        # Slots that are triggerd if the user changes something
        # ==================================================================================================================

    def _on_port_changed(self, port_string):
        self.model.port = port_string
        self.model.laser_config.port.set(port_string)
        self.logger.debug(f"Port changed to: {port_string}")
        # self._ui.cb_port_selection.setText(port_string)

    def _on_btn_connect_clicked(self):
        if self.model.connected:
            self.logger.debug("Attempting to disconnect from laser")
            self.controller.disconnect_device()
        elif not self.model.connected:
            self.logger.debug(f"Attempting to connect to laser on port: {self.model.port}")
            self.controller.connect_device(usb_port=self.model.port)


    def _on_btn_move_to_wavelength_clicked(self):
        self.logger.debug(f"Attempting to set wavelength to: {self._ui.sb_set_wavelength.value()}")
        self.controller.move_to_wavelength(
            self.model.port,
            self._ui.sb_set_wavelength.value())

    def _on_sb_sweep_start_changed(self, value):
        try:
            self.logger.debug(f"Sweep start changed. New Range {value}-{self.model.sweep_stop_wavelength}")
            self.model.sweep_start_wavelength = int(value)
        except Exception as e:
            self.logger.error(e)
            self.model.sweep_stop_wavelength = int(value)

    def _on_sb_sweep_stop_changed(self, value):
        try:
            self.logger.debug(f"Sweep stop changed. New Range {self.model.sweep_start_wavelength}-{value}")
            self.model.sweep_stop_wavelength = int(value)
        except Exception as e:
            self.logger.error(e)
            self.model.sweep_start_wavelength = int(value)
        # self.controller.set_sweep_wavelength_range(self._ui.sb_sweep_start.value(), self._ui.sb_sweep_stop.value())

    def _on_btn_start_sweep_clicked(self):
        self.logger.warning("Sweep manually started")
        self.controller.start_wavelength_sweep.emit(self.model.sweep_start_wavelength, self.model.sweep_stop_wavelength)

    def _on_sb_velocity_editing_finished(self):
        self.controller.set_velocity(self._ui.sb_velocity.value())

    def _on_sb_acc_editing_finished(self):
        self.controller.set_acceleration(self._ui.sb_acc.value())

    def _on_sb_dec_editing_finished(self):
        self.controller.set_deceleration(self._ui.sb_dec.value())

    def _on_btn_save_velocity_params_clicked(self):
        self.controller.set_velocity(self._ui.sb_velocity.value())
        self.controller.set_acceleration(self._ui.sb_acc.value())
        self.controller.set_deceleration(self._ui.sb_dec.value())

    def _display_estimated_progress(self, start_wavelength: float, stop_wavelength: float):

        # Dialog that displays the estimated time until the sweep is finished
        # calulate the time, the laser accelerates using
        self.laser_moving_progress_dialog.show()
        et_acc_dec = self.model.velocity / self.model.acceleration
        wl_after_acc = start_wavelength + 0.5 * self.model.acceleration * et_acc_dec ** 2
        wl_bevore_dec = stop_wavelength - 0.5 * self.model.deceleration * et_acc_dec ** 2
        if wl_bevore_dec > wl_after_acc:
            self.eta = 2 * et_acc_dec + (wl_bevore_dec - wl_after_acc) / self.model.velocity
        else:
            self.eta = 2 * et_acc_dec + (wl_after_acc - wl_bevore_dec) / self.model.velocity


        self.laser_moving_progress_dialog.setLabelText(
            f"Laser is moving {start_wavelength}-{stop_wavelength}."
            f"\nEstimated time until sweep is finished: {self.eta}")
        steps_per_second = self.eta / 100
        self.timer.start(steps_per_second * 1000)

    def _exit_display_estimated_progress(self):
        self.timer.stop()
        self.laser_moving_progress_dialog.close()

    def handle_timer(self):
        value = self.laser_moving_progress_dialog.value() + 1
        self.laser_moving_progress_dialog.setValue(value)

    def closeEvent(self, event):
        # do stuff
        self.controller.safe_exit()



import os
import pathlib
import sys
from ctypes import CDLL, cdll

sys.path.append(os.path.join(os.path.dirname(__file__), '../../'))

dir = pathlib.Path(f"{os.path.dirname(os.path.realpath(__file__))}")
dir = str(dir.resolve().as_posix())
print(dir)


# ======================================================================================================================
# EposCMD64.dll is needed, thus, try to copy it
# ======================================================================================================================
import os
import pathlib
import shutil



def copyEposDLL(epos_dll="./EposCMD64.dll", logger=None):
    if logger is None:
        logger = print
    else:
        logger = logger.info

    dll_dir = pathlib.Path(epos_dll)
    dll_name = os.path.basename(epos_dll)
    #epos_dll_name = pathlib.Path(epos_dll_name)
    logger(f"Copying {dll_dir} to {os.getcwd()}")

    if not dll_dir.exists():
         raise FileNotFoundError(f"Could not find {epos_dll}")

    dll_dir = str(dll_dir.resolve())
    dll_dest = f"{os.getcwd()}/{dll_name}"  # Copy the file to the workspace folder
    # Copy the file "EposCMD64.dll" to the current dir
    if not pathlib.Path(dll_dest).exists():
        shutil.copyfile(dll_dir, f"{os.getcwd()}/EposCMD64.dll")
        logger(f"Copied {dll_name} to {os.getcwd()}")
    elif pathlib.Path(dll_dest).exists():
        logger(f"{dll_name} already exists in {os.getcwd()}")


    # Check if the file was copied
    if not pathlib.Path(dll_dest).exists():
        raise FileNotFoundError(f"Could not copy {dll_name} to {os.getcwd()}")

def check_if_git_or_pip():
    # check if a folder .git is present ../../
    git_dir = pathlib.Path(__file__).parent.parent.parent / ".git"
    if git_dir.exists():
        print("git_dir:", git_dir)
    else:
        print("git_dir does not exist")

def get_pyprojecttoml() -> pathlib.Path:
    # is found in ../../pyconfig.toml
    pytoml_via_git = pathlib.Path(__file__).parent.parent.parent / "pyproject.toml"
    # found in ./pyconfig.toml: Copied to the root dir
    pytoml_via_pip = pathlib.Path(__file__).parent / "pyproject.toml"

    if pytoml_via_git.exists():
        #print("pytoml_via_git:", pytoml_via_git)
        return pytoml_via_git.resolve().absolute()
    elif pytoml_via_pip.exists():
        #print("pytoml_via_pip:", pytoml_via_pip)
        return pytoml_via_pip.resolve().absolute()




import logging
from multiprocessing import Value, Lock

#import ADScopeControl as captdev
import mpPy6
from PySide6.QtCore import Signal
from mpPy6.CProcessControl import CProcessControl

from SacherECLControl import Helpers
from SacherECLControl.model.LaserControlModel import LaserControlModel


class MPLaserDeviceControl(CProcessControl):
    connected_changed = Signal(bool, name='connected_changed')

    # Properties, automatically triggerd on a change by the process class
    current_wavelength_changed = Signal(float, name='current_wavelength_changed')
    min_wavelength_changed = Signal(float, name='min_wavelength_changed')
    max_wavelength_changed = Signal(float, name='max_wavelength_changed')
    velocity_changed = Signal(float, name='velocity_changed')
    acceleration_changed = Signal(float, name='acceleration_changed')
    deceleration_changed = Signal(float, name='deceleration_changed')

    ## Triggered when the laser settings are changed
    get_laser_settings_changed = Signal( name='get_laser_settings_changed')




    get_max_wavelength_finished = Signal(float, name='get_max_wavelength_finished')
    get_velocity_finished = Signal(float, name='get_velocity_finished')
    get_acceleration_finished = Signal(float, name='get_acceleration_finished')
    get_deceleration_finished = Signal(float, name='get_deceleration_finished')
    mp_read_laser_settings_finished = Signal(name='mp_read_laser_settings_finished')

    move_to_wavelength_finished = Signal(float, name='move_to_wavelength_finished')
    wavelength_sweep_finished = Signal(float, float, name='wavelength_sweep_finished')

    laser_is_moving_changed = Signal(bool, float, name='laser_is_moving_changed')
    wavelength_sweep_running_changed = Signal(bool, name='wavelength_sweep_running_changed')
    movement_finished_changed = Signal(bool, name='movement_finished_changed')

    start_wavelength_sweep = Signal(float, float, name='start_wavelength_sweep')

    def __init__(self, model: LaserControlModel,
                 start_capture_flag: Value, module_log=True, module_log_level=logging.WARNING):
        super().__init__(module_log=module_log, module_log_level=module_log_level)

        # This is a workaround, otherwise the dll and pyd object import would not work
        # Reason: We need assign the correct path beforehand. If we would first try to import the MPLaserDevice class
        # the pathes would net be set.
        # TODO: Find a better approach than this hacky way?
        from SacherECLControl.controller.multiprocess.MPLaserDevice import MPLaserDevice
        self.model = model

        self.lock = Lock()
        self._laser_moving_flag = Value('i', False, lock=self.lock)
        self._laser_finished_flag = Value('i', False, lock=self.lock)
        if start_capture_flag is None:
            start_capture_flag = Value('i', 0)

        self.register_child_process(MPLaserDevice,
                                    self._laser_moving_flag,
                                    self._laser_finished_flag,
                                    start_capture_flag,
                                    self.model.laser_config.epos_dll.get())

        self.connected_changed.connect(self._on_connected_changed)

        self.current_wavelength_changed.connect(
            lambda x: type(self.model).current_wavelength.fset(self.model, x))

        self.min_wavelength_changed.connect(
            lambda x: type(self.model).min_laser_wavelength.fset(self.model, x))

        self.max_wavelength_changed.connect(
            lambda x: type(self.model).max_laser_wavelength.fset(self.model, x))

        self.velocity_changed.connect(
            lambda x: type(self.model).velocity.fset(self.model, x))

        self.acceleration_changed.connect(
            lambda x: type(self.model).acceleration.fset(self.model, x))

        self.deceleration_changed.connect(
            lambda x: type(self.model).deceleration.fset(self.model, x))

        self.move_to_wavelength_finished.connect(self._move_to_wavelength_finished)

        self.laser_is_moving_changed.connect(self._laser_is_moving_changed)
        self.movement_finished_changed.connect(self._laser_movement_finished)
        self.wavelength_sweep_running_changed.connect(self._on_wavelength_sweep_running_changed)

        self.start_wavelength_sweep.connect(self._start_wavelength_sweep)
        self.kill_thread = False

    def _on_connected_changed(self, connected: bool):
        if connected:
            self.logger.info(f"[{connected}] Laser connection established.")
        else:
            self.logger.info(f"[{connected}] Laser disconnected.")
        self.model.connected = connected

    # ==================================================================================================================
    # Connection
    # ==================================================================================================================
    @mpPy6.CProcessControl.register_function()
    def connect_device(self, usb_port: str):
        """
        Establishes a connection to the Epos and reads all the needed data saved on the internal registry.
        :param usb_port: usb port on which the connection should be established.
        """
        self._module_logger.info(f"Connecting to process using {usb_port}")

    @mpPy6.CProcessControl.register_function()
    def disconnect_device(self):
        """
        Disconnects the Epos device/laser.
        """
        self._module_logger.info("Disconnecting from laser.")

    @mpPy6.CProcessControl.register_function()
    def set_velocity(self, velocity: float) -> None:
        """ Set the velocity of the laser. """
        self._module_logger.info(f"Setting velocity to {velocity}.")

    @mpPy6.CProcessControl.register_function()
    def set_acceleration(self, acceleration: float) -> None:
        """ Set the velocity of the laser. """
        self._module_logger.info(f"Setting acceleration to {acceleration}.")

    @mpPy6.CProcessControl.register_function()
    def set_deceleration(self, acceleration: float) -> None:
        """ Set the velocity of the laser. """
        self._module_logger.info(f"Setting acceleration to {acceleration}.")

    def set_start_capture_flag(self, start_capture_flag: Value):
        self._start_capture_flag = start_capture_flag

    @mpPy6.CProcessControl.register_function(connected_changed)
    def get_connected(self):
        self._module_logger.info("Reading current connection state from process.")

    @mpPy6.CProcessControl.register_function(get_laser_settings_changed)
    def get_laser_settings(self):
        self._module_logger.info("Reading current laser status.")

    @mpPy6.CProcessControl.register_function(move_to_wavelength_finished)
    def move_to_wavelength(self, usb_port: str, wavelength: float):
        self._module_logger.info(f"Moving laser ({usb_port}) to wavelength {wavelength} from process")

    @mpPy6.CProcessControl.register_function(wavelength_sweep_finished)
    def wavelength_sweep(self, usb_port: str, wavelength_start: float, wavelength_end: float):
        print(f"Sweeping laser ({usb_port}): Wavelength {wavelength_start} - {wavelength_end} from process")

    # ==================================================================================================================
    #
    # ==================================================================================================================
    def supervise(self, device):
        self.logger.info(f"Connecting to capture device for supervising. Device: {device.__class__.__name__}")
        if device.__class__.__name__ == "BaseADScopeController":
            self.model.capturing_device = device
            self.model.capturing_device.model.device_information.signals.device_connected_changed.connect(
                lambda x: type(self.model).capturing_device_connected.fset(self.model, x)
            )

            device.model.supervisor_information.supervised = True
            device.model.supervisor_information.supervisor = self


    def _move_to_wavelength_finished(self, wavelength: float):
        self.logger.info(f"Move to wavelength finished. Current wavelength: {wavelength}")


    def _laser_is_moving_changed(self, is_moving: bool, to_wavelength: float):
        self.model.laser_is_moving = (is_moving, to_wavelength)
        if is_moving:
            self.logger.info(f"************** Laser is moving: {is_moving}. "
                             f"Moving to {self.model.laser_moving_to_wavelength} nm")
        else:
            if is_moving:
                self.logger.info(f"************** Laser is moving: {is_moving}. "
                                 f"Stopped at {self.model.laser_moving_to_wavelength} nm")

    def _laser_movement_finished(self, is_finished: bool):
        pass

    def _on_wavelength_sweep_running_changed(self, running: bool):
        self.model.wavelength_sweep_running = running
        if running:
            self.logger.info(f"Wavelength sweep running.")
        else:
            self.logger.info(f"Wavelength sweep stopped.")

    def _start_wavelength_sweep(self, start_wavelength: float = None, stop_wavelength: float = None) -> None:
        self.model.wavelength_sweep_running = True
        self.logger.info(f"Starting wavelength sweep from {start_wavelength} to {stop_wavelength}")
        if self.model.capturing_device is not None:
            self.model.capturing_device.reset_capture()
            if not self.model.capturing_device.model.device_information.device_connected:
                # self.model.
                self.model.capturing_device.open_device()
                self.model.capturing_device.ready_for_recording_changed.connect(
                    lambda ready: self.start_wavelength_sweep_emitted(start_wavelength, stop_wavelength, ready=ready)
                )
            self.logger.info("Capturing device is connected.")

            if self.model.capturing_device.model.capturing_information.ready_for_recording:
                self.start_wavelength_sweep_emitted(start_wavelength, stop_wavelength)
        else:
            self.start_wavelength_sweep_emitted(start_wavelength, stop_wavelength)

    def start_wavelength_sweep_emitted(self, start_wavelength: float = None, stop_wavelength: float = None, ready=True):
        if ready:
            if start_wavelength is None:
                start_wavelength = self.model.sweep_start_wavelength
            if stop_wavelength is None:
                stop_wavelength = self.model.sweep_stop_wavelength
            self.wavelength_sweep(self.model.port, start_wavelength, stop_wavelength)



class LaserScelton():

    # ==================================================================================================================
    # Connection functions
    # ==================================================================================================================
    def connectMotor(self, port: str = "USB0") -> None:
        """
        Connects EPOS motor at the given port
        :param port: Port of the USB connect (USB0, USB1, USB2, USB3, ...)
        """
        raise NotImplementedError()

    def closeMotorConnection(self) -> None:
        """
        Closes EPOS motor connection
        :return:
        """
        raise NotImplementedError()

    # ==================================================================================================================
    # Motor Position functions
    # ==================================================================================================================
    def getcurrentWavelengthPositionIs(self) -> float:
        """
        Returns calculated wavelength in nm. This function returns the actual wavelength calculated from the
        real motor position (not the value stored in the motor memory).
        Returns the current encoder position and is therefore a stronger indication that the motor is actually moving.
        Values of "getCurrentWavelength" and "getcurrentWavelengthPositionIs" can differ between 1 to 2 positions,
        after the calibration has been performed.
        :return: calculated wavelength in nm (float)
       """
        raise NotImplementedError()

    def getCurrentWavelength(self) -> float:
        """
        Returns stored wavelength in nm.
        :return: stored wavelength in nm (float)
        """
        raise NotImplementedError()

    def setVelocity(self, velocity: float) -> None:
        """
        Sets the velocity in nm/s
        :param velocity: velocity in nm/s
        :return: None
        """
        raise NotImplementedError()

    def getVelocity(self) -> float:
        """
        Returns velocity in nm/s
        :return:
        """
        raise NotImplementedError()

    def setAcceleration(self, acceleration: float) -> None:
        """
        Sets the acceleration in nm/s^2
        :param acceleration:
        :return:
        """
        raise NotImplementedError()

    def getAcceleration(self) -> float:
        """
        Returns acceleration in nm/s^2
        :return:
        """
        raise NotImplementedError()

    def setDeceleration(self, deceleration: float) -> None:
        """
        Sets the deceleration in nm/s^2
        :param deceleration:
        :return:
        """
        raise NotImplementedError()

    def getDeceleration(self) -> float:
        """
        Returns deceleration in nm/s^2
        :return:
        """
        raise NotImplementedError()

    def getStoredPosition(self, isWvl: bool = True) -> float:
        """
        Returns stored motor position in steps or in nm.
        :param isWvl: returns store position in nm if True, else in steps
        :return: motor position in steps or in nm
        """
        raise NotImplementedError()

    def getOffset(self) -> float:
        """
        Returns difference between stored and current motor positions.
        :return:
        """
        raise NotImplementedError()

    def getPositionIs(self) -> float:
        """
        Returns current (not stored) motor position in steps.
        :return: current motor position in steps
        """
        raise NotImplementedError()

    def resetPositionIs(self) -> None:
        """
        Moves motor to position 0.
        :return: None
        """
        raise NotImplementedError()
    # ==================================================================================================================
    # Motor movement functions
    # ==================================================================================================================
    def goToWvl(self, wavelength: float, fast: bool) -> None:
        """
        Moves motor to the given wavelength in nm.
        :param wavelength: wavelength in nm
        :param fast: if True, moves motor with fast velocity, else with slow velocity
        :return: None
        """
        raise NotImplementedError()

    def startScan(self, wavelength: float, target: float) -> None:
        """
        Start scan from StartWvl to Target
        :param wavelength: start wavelength in nm
        :param target: target wavelength in nm
        :return: None
        """
        raise NotImplementedError()

    # ==================================================================================================================
    # Calibration functions#
    # ==================================================================================================================
    def Calibrate(self, wavelength: float) -> None:
        """
        Calibrates the system with wavelength
        :param wavelength: wavelength in nm
        :return:
        """
        raise NotImplementedError()

    # ==================================================================================================================
    # Auxiliary functions, which read or store variables in the code and on the EPOS board.
    # ==================================================================================================================
    def setStartWvl(self, wavelength: float) -> None:
        """
        Sets the start wavelength in nm. Auxiliary functions, which read or store variables in the code and on the
        EPOS board.
        :param wavelength: start wavelength in nm
        :return: None
        """
        raise NotImplementedError()

    def getStartWvl(self) -> float:
        """
        Returns start wavelength in nm. Auxiliary functions, which read or store variables in the code and on the
        EPOS board.
        :return: start wavelength in nm (float)
        """
        raise NotImplementedError()

    def setTargetWvl(self, wavelength: float) -> None:
        """
        Sets the target wavelength in nm. Auxiliary functions, which read or store variables in the code and on the
        EPOS board.
        :param wavelength:
        :return:
        """
        raise NotImplementedError()

    def getTargetWvl(self) -> float:
        """
        Returns target wavelength in nm. Auxiliary functions, which read or store variables in the code and on the
        EPOS board.
        :return: target wavelength in nm (float)
        """
        raise NotImplementedError()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'LaserControlWindow.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractSpinBox, QApplication, QComboBox, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QHBoxLayout,
    QLCDNumber, QLabel, QMainWindow, QPushButton,
    QSizePolicy, QSlider, QTabWidget, QToolButton,
    QVBoxLayout, QWidget)
from . import resources_rc

class Ui_LaserControlWindow(object):
    def setupUi(self, LaserControlWindow):
        if not LaserControlWindow.objectName():
            LaserControlWindow.setObjectName(u"LaserControlWindow")
        LaserControlWindow.resize(697, 717)
        self.centralwidget = QWidget(LaserControlWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.centralwidget.setStyleSheet(u"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"\n"
"SET APP STYLESHEET - FULL STYLES HERE\n"
"DARK THEME - DRACULA COLOR BASED\n"
"\n"
"///////////////////////////////////////////////////////////////////////////////////////////////// */\n"
"QWidget{\n"
"    background-color: rgb(40, 44, 52);\n"
"	color: rgb(221, 221, 221);\n"
"	font: 10pt \"Segoe UI\";\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Tooltip */\n"
"QToolTip {\n"
"	color: #ffffff;\n"
"	background-color: rgba(33, 37, 43, 180);\n"
"	border: 1px solid rgb(44, 49, 58);\n"
"	background-image: none;\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 2px solid rgb(255, 121, 198);\n"
"	text-align: left;\n"
"	padding-left: 8px;\n"
"	margin: 0px;\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Q"
                        "TableWidget */\n"
"QTableWidget {	\n"
"	background-color: transparent;\n"
"	padding: 10px;\n"
"	border-radius: 5px;\n"
"	gridline-color: rgb(44, 49, 58);\n"
"	border-bottom: 1px solid rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::item{\n"
"	border-color: rgb(44, 49, 60);\n"
"	padding-left: 5px;\n"
"	padding-right: 5px;\n"
"	gridline-color: rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::item:selected{\n"
"	background-color: rgb(189, 147, 249);\n"
"}\n"
"QHeaderView::section{\n"
"	background-color: rgb(33, 37, 43);\n"
"	max-width: 30px;\n"
"	border: 1px solid rgb(44, 49, 58);\n"
"	border-style: none;\n"
"    border-bottom: 1px solid rgb(44, 49, 60);\n"
"    border-right: 1px solid rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::horizontalHeader {	\n"
"	background-color: rgb(33, 37, 43);\n"
"}\n"
"QHeaderView::section:horizontal\n"
"{\n"
"    border: 1px solid rgb(33, 37, 43);\n"
"	background-color: rgb(33, 37, 43);\n"
"	padding: 3px;\n"
"	border-top-left-radius: 7px;\n"
"    border-top-right-radius: 7px;\n"
"}\n"
"QHeaderView::se"
                        "ction:vertical\n"
"{\n"
"    border: 1px solid rgb(44, 49, 60);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"LineEdit */\n"
"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"PlainTextEdit */\n"
"QPlainTextEdit {\n"
"	background-color: rgb(27, 29, 35);\n"
"	border-radius: 5px;\n"
"	padding: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QPlainTextEdit  QScrollBar:vertical {\n"
"    width: 8px;\n"
" }\n"
"QPlainTextEdit  QScrollBar:horizo"
                        "ntal {\n"
"    height: 8px;\n"
" }\n"
"QPlainTextEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QPlainTextEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"ScrollBars */\n"
"QScrollBar:horizontal {\n"
"    border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    height: 8px;\n"
"    margin: 0px 21px 0 21px;\n"
"	border-radius: 0px;\n"
"}\n"
"QScrollBar::handle:horizontal {\n"
"    background: rgb(189, 147, 249);\n"
"    min-width: 25px;\n"
"	border-radius: 4px\n"
"}\n"
"QScrollBar::add-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-right-radius: 4px;\n"
"    border-bottom-right-radius: 4px;\n"
"    subcontrol-position: right;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QScrollBar::sub-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-left-r"
                        "adius: 4px;\n"
"    border-bottom-left-radius: 4px;\n"
"    subcontrol-position: left;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
"QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
" QScrollBar:vertical {\n"
"	border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    width: 8px;\n"
"    margin: 21px 0 21px 0;\n"
"	border-radius: 0px;\n"
" }\n"
" QScrollBar::handle:vertical {	\n"
"	background: rgb(189, 147, 249);\n"
"    min-height: 25px;\n"
"	border-radius: 4px\n"
" }\n"
" QScrollBar::add-line:vertical {\n"
"     border: none;\n"
"    background: rgb(55, 63, 77);\n"
"     height: 20px;\n"
"	border-bottom-left-radius: 4px;\n"
"    border-bottom-right-radius: 4px;\n"
"     subcontrol-position: bottom;\n"
"     subcontrol-origin: margin;\n"
" }\n"
" QScrollBar::sub-line:vertical {\n"
"	border: none;\n"
"    background: rgb(55, 63, 77);\n"
"     he"
                        "ight: 20px;\n"
"	border-top-left-radius: 4px;\n"
"    border-top-right-radius: 4px;\n"
"     subcontrol-position: top;\n"
"     subcontrol-origin: margin;\n"
" }\n"
" QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
" QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"CheckBox */\n"
"QCheckBox::indicator {\n"
"    border: 3px solid rgb(52, 59, 72);\n"
"	width: 15px;\n"
"	height: 15px;\n"
"	border-radius: 10px;\n"
"    background: rgb(44, 49, 60);\n"
"}\n"
"QCheckBox::indicator:hover {\n"
"    border: 3px solid rgb(58, 66, 81);\n"
"}\n"
"QCheckBox::indicator:checked {\n"
"    background: 3px solid rgb(52, 59, 72);\n"
"	border: 3px solid rgb(52, 59, 72);	\n"
"	background-image: url(:/icons/images/icons/cil-check-alt.png);\n"
"}\n"
"\n"
"/* ///////////////////////////////////////////////////////////"
                        "//////////////////////////////////////\n"
"RadioButton */\n"
"QRadioButton::indicator {\n"
"    border: 3px solid rgb(52, 59, 72);\n"
"	width: 15px;\n"
"	height: 15px;\n"
"	border-radius: 10px;\n"
"    background: rgb(44, 49, 60);\n"
"}\n"
"QRadioButton::indicator:hover {\n"
"    border: 3px solid rgb(58, 66, 81);\n"
"}\n"
"QRadioButton::indicator:checked {\n"
"    background: 3px solid rgb(94, 106, 130);\n"
"	border: 3px solid rgb(52, 59, 72);	\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"ComboBox */\n"
"QComboBox{\n"
"	background-color: rgb(52, 59, 72);\n"
"	border-radius: 2px;\n"
"	border: 1px solid rgb(0, 0, 0);\n"
"}\n"
"QComboBox:hover{\n"
"	border: 1px solid rgb(42, 175, 211);\n"
"}\n"
"QComboBox::drop-down {\n"
"	subcontrol-origin: padding;\n"
"	subcontrol-position: top right;\n"
"	width: 25px; \n"
"	border-left-width: 2px;\n"
"	border-left-color: rgba(39, 44, 54, 150);\n"
"	border-left-style: solid;\n"
"	border-top-right-radius"
                        ": 3px;\n"
"	border-bottom-right-radius: 3px;	\n"
"	background-image: url(:/icons/icons/cil-arrow-bottom.png);\n"
"	background-position: center;\n"
"	background-repeat: no-reperat;\n"
" }\n"
"QComboBox QAbstractItemView {\n"
"	color: rgb(255, 121, 198);	\n"
"	background-color: rgb(33, 37, 43);\n"
"	padding: 10px;\n"
"	selection-background-color: rgb(39, 44, 54);\n"
"}\n"
"/*QComboBox QAbstractItemView::item {\n"
"  min-height: 150px;\n"
"}*/\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Sliders */\n"
"QSlider::groove:horizontal {\n"
"    border-radius: 5px;\n"
"    height: 10px;\n"
"	margin: 0px;\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QSlider::groove:horizontal:hover {\n"
"	background-color: rgb(55, 62, 76);\n"
"}\n"
"QSlider::handle:horizontal {\n"
"    background-color: rgb(0, 0, 0);\n"
"    border: 1px solid rgb(42, 175, 211);\n"
"    height: 10px;\n"
"    width: 8px;\n"
"    margin: 0px;\n"
"	border-radius: 5px;\n"
"}\n"
"QSlider"
                        "::handle:horizontal:hover {\n"
"    background-color: rgb(42, 141, 211);\n"
"    border: none;\n"
"    height: 10px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	border-radius: 5px;\n"
"}\n"
"QSlider::handle:horizontal:pressed {\n"
"    background-color: rgb(42, 141, 211);\n"
"}\n"
"\n"
"QSlider::groove:vertical {\n"
"    border-radius: 5px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QSlider::groove:vertical:hover {\n"
"	background-color: rgb(55, 62, 76);\n"
"}\n"
"QSlider::handle:vertical {\n"
"    background-color: rgb(189, 147, 249);\n"
"	border: none;\n"
"    height: 10px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	border-radius: 5px;\n"
"}\n"
"QSlider::handle:vertical:hover {\n"
"    background-color: rgb(195, 155, 255);\n"
"}\n"
"QSlider::handle:vertical:pressed {\n"
"    background-color: rgb(255, 121, 198);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"CommandLinkButton */\n"
""
                        "QCommandLinkButton {	\n"
"	color: rgb(255, 121, 198);\n"
"	border-radius: 5px;\n"
"	padding: 5px;\n"
"	color: rgb(255, 170, 255);\n"
"}\n"
"QCommandLinkButton:hover {	\n"
"	color: rgb(255, 170, 255);\n"
"	background-color: rgb(44, 49, 60);\n"
"}\n"
"QCommandLinkButton:pressed {	\n"
"	color: rgb(189, 147, 249);\n"
"	background-color: rgb(52, 58, 71);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Button */\n"
"QPushButton {\n"
"	border: 1px solid rgb(42, 175, 211);\n"
"	border-radius: 2px;	\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QPushButton:hover {\n"
"	border: 1px solid rgb(42, 141, 211);\n"
"    border-radius: 2px;	\n"
"	background-color: rgb(42, 141, 211);\n"
"}\n"
"QPushButton:pressed {	\n"
"	border: 1px solid rgb(42, 141, 211);\n"
"    border-radius: 2px;	\n"
"	background-color: rgb(35, 40, 49);\n"
"}\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"\n"
"/* QMenu ------------------------------------------------------------------"
                        "\n"
"\n"
"examples: https://doc.qt.io/qt-5/stylesheet-examples.html#customizing-qmenu\n"
"\n"
"--------------------------------------------------------------------------- */\n"
"QMenu {\n"
"    background-color: rgb(40, 44, 52);\n"
"    margin: 2px; /* some spacing around the menu */\n"
"}\n"
"\n"
"QMenu::item {\n"
"    padding: 2px 25px 2px 20px;\n"
"    border: 1px solid transparent; /* reserve space for selection border */\n"
"}\n"
"\n"
"QMenu::item:selected {\n"
"    border-color: darkblue;\n"
"    background: rgba(100, 100, 100, 150);\n"
"}\n"
"\n"
"QMenu::icon:checked { /* appearance of a 'checked' icon */\n"
"    background: gray;\n"
"    border: 1px inset gray;\n"
"    position: absolute;\n"
"    top: 1px;\n"
"    right: 1px;\n"
"    bottom: 1px;\n"
"    left: 1px;\n"
"}\n"
"\n"
"QMenu::separator {\n"
"    height: 2px;\n"
"    background: lightblue;\n"
"    margin-left: 10px;\n"
"    margin-right: 5px;\n"
"}\n"
"\n"
"QMenu::indicator {\n"
"    width: 13px;\n"
"    height: 13px;\n"
"}\n"
"\n"
"QTabWidge"
                        "t::pane {\n"
"  border: 1px solid lightgray;\n"
"  top:-1px; \n"
"  background:  rgb(40, 44, 52); \n"
"} \n"
"\n"
"QTabBar::tab {\n"
"  background: rgb(40, 44, 52);; \n"
"  border: 1px solid lightgray; \n"
"  padding: 2px;\n"
"	padding-left: 10px;\n"
"	padding-right: 10px;\n"
"} \n"
"\n"
"QTabBar::tab:selected { \n"
"  background:  rgb(189, 147, 249);\n"
"  margin-bottom: -1px; \n"
"}\n"
"")
        self.gridLayout_3 = QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.table_laser_control = QTabWidget(self.centralwidget)
        self.table_laser_control.setObjectName(u"table_laser_control")
        self.tab_control = QWidget()
        self.tab_control.setObjectName(u"tab_control")
        self.gridLayout_9 = QGridLayout(self.tab_control)
        self.gridLayout_9.setObjectName(u"gridLayout_9")
        self.grd_tab_settings = QGridLayout()
        self.grd_tab_settings.setObjectName(u"grd_tab_settings")
        self.group_motor_profile = QGroupBox(self.tab_control)
        self.group_motor_profile.setObjectName(u"group_motor_profile")
        self.group_motor_profile.setEnabled(True)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.group_motor_profile.sizePolicy().hasHeightForWidth())
        self.group_motor_profile.setSizePolicy(sizePolicy)
        self.group_motor_profile.setMinimumSize(QSize(0, 0))
        self.group_motor_profile.setMaximumSize(QSize(16777215, 150))
        self.gridLayout_7 = QGridLayout(self.group_motor_profile)
        self.gridLayout_7.setObjectName(u"gridLayout_7")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.gridLayout_11 = QGridLayout()
        self.gridLayout_11.setObjectName(u"gridLayout_11")
        self.gridLayout_11.setContentsMargins(-1, -1, 0, 0)
        self.lbl_vel = QLabel(self.group_motor_profile)
        self.lbl_vel.setObjectName(u"lbl_vel")

        self.gridLayout_11.addWidget(self.lbl_vel, 0, 0, 1, 1)

        self.lbl_acc = QLabel(self.group_motor_profile)
        self.lbl_acc.setObjectName(u"lbl_acc")

        self.gridLayout_11.addWidget(self.lbl_acc, 1, 0, 1, 1)

        self.sb_velocity = QDoubleSpinBox(self.group_motor_profile)
        self.sb_velocity.setObjectName(u"sb_velocity")
        self.sb_velocity.setEnabled(True)
        self.sb_velocity.setCorrectionMode(QAbstractSpinBox.CorrectionMode.CorrectToNearestValue)

        self.gridLayout_11.addWidget(self.sb_velocity, 0, 1, 1, 1)

        self.sb_acc = QDoubleSpinBox(self.group_motor_profile)
        self.sb_acc.setObjectName(u"sb_acc")
        self.sb_acc.setEnabled(True)
        self.sb_acc.setCorrectionMode(QAbstractSpinBox.CorrectionMode.CorrectToNearestValue)

        self.gridLayout_11.addWidget(self.sb_acc, 1, 1, 1, 1)

        self.lbl_dec = QLabel(self.group_motor_profile)
        self.lbl_dec.setObjectName(u"lbl_dec")

        self.gridLayout_11.addWidget(self.lbl_dec, 2, 0, 1, 1)

        self.sb_dec = QDoubleSpinBox(self.group_motor_profile)
        self.sb_dec.setObjectName(u"sb_dec")
        self.sb_dec.setEnabled(True)
        self.sb_dec.setCorrectionMode(QAbstractSpinBox.CorrectionMode.CorrectToNearestValue)

        self.gridLayout_11.addWidget(self.sb_dec, 2, 1, 1, 1)


        self.verticalLayout_2.addLayout(self.gridLayout_11)


        self.gridLayout_7.addLayout(self.verticalLayout_2, 0, 0, 1, 1)


        self.grd_tab_settings.addWidget(self.group_motor_profile, 3, 0, 1, 2)

        self.group_wavelength_settings = QGroupBox(self.tab_control)
        self.group_wavelength_settings.setObjectName(u"group_wavelength_settings")
        self.group_wavelength_settings.setMinimumSize(QSize(0, 210))
        self.group_wavelength_settings.setMaximumSize(QSize(16777215, 300))
        self.gridLayout_5 = QGridLayout(self.group_wavelength_settings)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_12 = QGridLayout()
        self.gridLayout_12.setObjectName(u"gridLayout_12")
        self.lbl_wavelength = QLabel(self.group_wavelength_settings)
        self.lbl_wavelength.setObjectName(u"lbl_wavelength")

        self.gridLayout_12.addWidget(self.lbl_wavelength, 3, 0, 1, 1)

        self.btn_move_to_wavelength = QPushButton(self.group_wavelength_settings)
        self.btn_move_to_wavelength.setObjectName(u"btn_move_to_wavelength")

        self.gridLayout_12.addWidget(self.btn_move_to_wavelength, 5, 0, 1, 2)

        self.sb_set_wavelength = QDoubleSpinBox(self.group_wavelength_settings)
        self.sb_set_wavelength.setObjectName(u"sb_set_wavelength")
        self.sb_set_wavelength.setMinimumSize(QSize(150, 0))

        self.gridLayout_12.addWidget(self.sb_set_wavelength, 3, 1, 1, 1)

        self.lcd_wavelength = QLCDNumber(self.group_wavelength_settings)
        self.lcd_wavelength.setObjectName(u"lcd_wavelength")
        self.lcd_wavelength.setEnabled(True)
        self.lcd_wavelength.setMinimumSize(QSize(0, 62))
        self.lcd_wavelength.setMaximumSize(QSize(16777215, 62))
        self.lcd_wavelength.setFrameShape(QFrame.Shape.Box)
        self.lcd_wavelength.setFrameShadow(QFrame.Shadow.Sunken)
        self.lcd_wavelength.setLineWidth(1)
        self.lcd_wavelength.setMidLineWidth(0)
        self.lcd_wavelength.setSmallDecimalPoint(True)
        self.lcd_wavelength.setDigitCount(10)
        self.lcd_wavelength.setSegmentStyle(QLCDNumber.SegmentStyle.Flat)
        self.lcd_wavelength.setProperty("value", 830.000000000000000)
        self.lcd_wavelength.setProperty("intValue", 830)

        self.gridLayout_12.addWidget(self.lcd_wavelength, 9, 0, 1, 2)

        self.gridWidget = QWidget(self.group_wavelength_settings)
        self.gridWidget.setObjectName(u"gridWidget")
        self.gridWidget.setMaximumSize(QSize(16777215, 50))
        self.gridLayout_2 = QGridLayout(self.gridWidget)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.lbl_max_wavelength = QLabel(self.gridWidget)
        self.lbl_max_wavelength.setObjectName(u"lbl_max_wavelength")
        self.lbl_max_wavelength.setMaximumSize(QSize(16777215, 18))
        self.lbl_max_wavelength.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
        self.lbl_max_wavelength.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.gridLayout_2.addWidget(self.lbl_max_wavelength, 1, 1, 1, 1)

        self.lbl_min_wavelength = QLabel(self.gridWidget)
        self.lbl_min_wavelength.setObjectName(u"lbl_min_wavelength")
        self.lbl_min_wavelength.setMaximumSize(QSize(16777215, 18))
        self.lbl_min_wavelength.setMargin(0)

        self.gridLayout_2.addWidget(self.lbl_min_wavelength, 1, 0, 1, 1)

        self.slider_wavelength = QSlider(self.gridWidget)
        self.slider_wavelength.setObjectName(u"slider_wavelength")
        self.slider_wavelength.setMinimumSize(QSize(0, 10))
        self.slider_wavelength.setOrientation(Qt.Orientation.Horizontal)
        self.slider_wavelength.setInvertedControls(False)
        self.slider_wavelength.setTickPosition(QSlider.TickPosition.NoTicks)

        self.gridLayout_2.addWidget(self.slider_wavelength, 0, 0, 1, 2)


        self.gridLayout_12.addWidget(self.gridWidget, 6, 0, 1, 2)


        self.gridLayout_5.addLayout(self.gridLayout_12, 0, 0, 1, 1)


        self.grd_tab_settings.addWidget(self.group_wavelength_settings, 1, 0, 1, 2)

        self.group_sweep_settings = QGroupBox(self.tab_control)
        self.group_sweep_settings.setObjectName(u"group_sweep_settings")
        self.group_sweep_settings.setMinimumSize(QSize(250, 0))
        self.group_sweep_settings.setMaximumSize(QSize(16777215, 90))
        self.gridLayout_8 = QGridLayout(self.group_sweep_settings)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_10 = QGridLayout()
        self.gridLayout_10.setObjectName(u"gridLayout_10")
        self.sb_sweep_stop = QDoubleSpinBox(self.group_sweep_settings)
        self.sb_sweep_stop.setObjectName(u"sb_sweep_stop")

        self.gridLayout_10.addWidget(self.sb_sweep_stop, 0, 3, 1, 1)

        self.lbl_sweep_start = QLabel(self.group_sweep_settings)
        self.lbl_sweep_start.setObjectName(u"lbl_sweep_start")
        self.lbl_sweep_start.setMinimumSize(QSize(100, 0))

        self.gridLayout_10.addWidget(self.lbl_sweep_start, 0, 0, 1, 1)

        self.lbl_sweep_stop = QLabel(self.group_sweep_settings)
        self.lbl_sweep_stop.setObjectName(u"lbl_sweep_stop")
        self.lbl_sweep_stop.setMinimumSize(QSize(100, 0))

        self.gridLayout_10.addWidget(self.lbl_sweep_stop, 0, 2, 1, 1)

        self.btn_start_sweep = QPushButton(self.group_sweep_settings)
        self.btn_start_sweep.setObjectName(u"btn_start_sweep")

        self.gridLayout_10.addWidget(self.btn_start_sweep, 1, 0, 1, 4)

        self.sb_sweep_start = QDoubleSpinBox(self.group_sweep_settings)
        self.sb_sweep_start.setObjectName(u"sb_sweep_start")

        self.gridLayout_10.addWidget(self.sb_sweep_start, 0, 1, 1, 1)


        self.gridLayout_8.addLayout(self.gridLayout_10, 1, 0, 1, 1)


        self.grd_tab_settings.addWidget(self.group_sweep_settings, 2, 0, 1, 2)

        self.grp_system_info = QGroupBox(self.tab_control)
        self.grp_system_info.setObjectName(u"grp_system_info")
        self.grp_system_info.setMinimumSize(QSize(0, 110))
        self.grp_system_info.setMaximumSize(QSize(16777215, 110))
        self.gridLayout_6 = QGridLayout(self.grp_system_info)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.grd_system_info = QGridLayout()
        self.grd_system_info.setObjectName(u"grd_system_info")

        self.gridLayout_6.addLayout(self.grd_system_info, 0, 0, 1, 1)


        self.grd_tab_settings.addWidget(self.grp_system_info, 0, 1, 1, 1)

        self.grp_Connection = QGroupBox(self.tab_control)
        self.grp_Connection.setObjectName(u"grp_Connection")
        self.grp_Connection.setMinimumSize(QSize(250, 100))
        self.grp_Connection.setMaximumSize(QSize(300, 110))
        self.gridLayout_4 = QGridLayout(self.grp_Connection)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.vgrd_connection_settings = QVBoxLayout()
        self.vgrd_connection_settings.setObjectName(u"vgrd_connection_settings")
        self.lbl_port_selection = QLabel(self.grp_Connection)
        self.lbl_port_selection.setObjectName(u"lbl_port_selection")
        self.lbl_port_selection.setMaximumSize(QSize(16777215, 12))

        self.vgrd_connection_settings.addWidget(self.lbl_port_selection)

        self.cb_port_selection = QComboBox(self.grp_Connection)
        self.cb_port_selection.setObjectName(u"cb_port_selection")

        self.vgrd_connection_settings.addWidget(self.cb_port_selection)

        self.btn_connect = QPushButton(self.grp_Connection)
        self.btn_connect.setObjectName(u"btn_connect")

        self.vgrd_connection_settings.addWidget(self.btn_connect)


        self.gridLayout_4.addLayout(self.vgrd_connection_settings, 0, 0, 1, 1)


        self.grd_tab_settings.addWidget(self.grp_Connection, 0, 0, 1, 1)


        self.gridLayout_9.addLayout(self.grd_tab_settings, 0, 0, 1, 1)

        self.table_laser_control.addTab(self.tab_control, "")
        self.tab_uncertainty = QWidget()
        self.tab_uncertainty.setObjectName(u"tab_uncertainty")
        self.grd_uncertainty = QGridLayout(self.tab_uncertainty)
        self.grd_uncertainty.setObjectName(u"grd_uncertainty")
        self.table_laser_control.addTab(self.tab_uncertainty, "")
        self.tab_uncertainty_dist = QWidget()
        self.tab_uncertainty_dist.setObjectName(u"tab_uncertainty_dist")
        self.grd_uncertainty_dist = QGridLayout(self.tab_uncertainty_dist)
        self.grd_uncertainty_dist.setObjectName(u"grd_uncertainty_dist")
        self.table_laser_control.addTab(self.tab_uncertainty_dist, "")

        self.gridLayout_3.addWidget(self.table_laser_control, 2, 0, 1, 1)

        self.horizontalFrame = QFrame(self.centralwidget)
        self.horizontalFrame.setObjectName(u"horizontalFrame")
        self.horizontalFrame.setLayoutDirection(Qt.LayoutDirection.LeftToRight)
        self.horizontalLayout = QHBoxLayout(self.horizontalFrame)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.menuBar = QFrame(self.horizontalFrame)
        self.menuBar.setObjectName(u"menuBar")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.menuBar.sizePolicy().hasHeightForWidth())
        self.menuBar.setSizePolicy(sizePolicy1)
        self.menuBar.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.menuBar)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.menu_file = QToolButton(self.menuBar)
        self.menu_file.setObjectName(u"menu_file")

        self.horizontalLayout_2.addWidget(self.menu_file)

        self.menu_edit = QToolButton(self.menuBar)
        self.menu_edit.setObjectName(u"menu_edit")

        self.horizontalLayout_2.addWidget(self.menu_edit)

        self.menu_run = QToolButton(self.menuBar)
        self.menu_run.setObjectName(u"menu_run")

        self.horizontalLayout_2.addWidget(self.menu_run)


        self.horizontalLayout.addWidget(self.menuBar)

        self.horizontalFrame1 = QFrame(self.horizontalFrame)
        self.horizontalFrame1.setObjectName(u"horizontalFrame1")
        self.horizontalLayout_3 = QHBoxLayout(self.horizontalFrame1)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")

        self.horizontalLayout.addWidget(self.horizontalFrame1)


        self.gridLayout_3.addWidget(self.horizontalFrame, 0, 0, 1, 1)

        LaserControlWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(LaserControlWindow)

        self.table_laser_control.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(LaserControlWindow)
    # setupUi

    def retranslateUi(self, LaserControlWindow):
        LaserControlWindow.setWindowTitle(QCoreApplication.translate("LaserControlWindow", u"MainWindow", None))
        self.group_motor_profile.setTitle(QCoreApplication.translate("LaserControlWindow", u"Motor Profile", None))
        self.lbl_vel.setText(QCoreApplication.translate("LaserControlWindow", u"Velocity", None))
        self.lbl_acc.setText(QCoreApplication.translate("LaserControlWindow", u"Acceleration", None))
        self.sb_velocity.setSuffix(QCoreApplication.translate("LaserControlWindow", u" nm/s", None))
        self.sb_acc.setSuffix(QCoreApplication.translate("LaserControlWindow", u" nm/s", None))
        self.lbl_dec.setText(QCoreApplication.translate("LaserControlWindow", u"Deceleration", None))
        self.sb_dec.setSuffix(QCoreApplication.translate("LaserControlWindow", u" nm/s", None))
        self.group_wavelength_settings.setTitle(QCoreApplication.translate("LaserControlWindow", u"Wavelength", None))
        self.lbl_wavelength.setText(QCoreApplication.translate("LaserControlWindow", u"Wavelength [nm]", None))
        self.btn_move_to_wavelength.setText(QCoreApplication.translate("LaserControlWindow", u"Move", None))
        self.sb_set_wavelength.setSuffix(QCoreApplication.translate("LaserControlWindow", u"nm", None))
        self.lbl_max_wavelength.setText(QCoreApplication.translate("LaserControlWindow", u"870 nm", None))
        self.lbl_min_wavelength.setText(QCoreApplication.translate("LaserControlWindow", u"830 nm", None))
        self.group_sweep_settings.setTitle(QCoreApplication.translate("LaserControlWindow", u"Sweep Settings", None))
        self.sb_sweep_stop.setSuffix(QCoreApplication.translate("LaserControlWindow", u" nm", None))
        self.lbl_sweep_start.setText(QCoreApplication.translate("LaserControlWindow", u"Sweep Start [nm]", None))
        self.lbl_sweep_stop.setText(QCoreApplication.translate("LaserControlWindow", u"Sweep Stop [nm]", None))
        self.btn_start_sweep.setText(QCoreApplication.translate("LaserControlWindow", u"Start Sweep", None))
        self.sb_sweep_start.setSuffix(QCoreApplication.translate("LaserControlWindow", u" nm", u"nm/s"))
        self.grp_system_info.setTitle(QCoreApplication.translate("LaserControlWindow", u"System Information", None))
        self.grp_Connection.setTitle(QCoreApplication.translate("LaserControlWindow", u"Connection Settings", None))
        self.lbl_port_selection.setText(QCoreApplication.translate("LaserControlWindow", u"USB Port Connection", None))
        self.btn_connect.setText(QCoreApplication.translate("LaserControlWindow", u"Connect", None))
        self.table_laser_control.setTabText(self.table_laser_control.indexOf(self.tab_control), QCoreApplication.translate("LaserControlWindow", u"Laser Control", None))
        self.table_laser_control.setTabText(self.table_laser_control.indexOf(self.tab_uncertainty), QCoreApplication.translate("LaserControlWindow", u"Uncertainty", None))
        self.table_laser_control.setTabText(self.table_laser_control.indexOf(self.tab_uncertainty_dist), QCoreApplication.translate("LaserControlWindow", u"Uncertainty Distribution", None))
        self.menu_file.setText(QCoreApplication.translate("LaserControlWindow", u"File", None))
        self.menu_edit.setText(QCoreApplication.translate("LaserControlWindow", u"Edit", None))
        self.menu_run.setText(QCoreApplication.translate("LaserControlWindow", u"Run", None))
    # retranslateUi




from PySide6.QtWidgets import QGridLayout, QGroupBox, QLabel, QWidget

from WidgetCollection.widgets.LEDIndicatorWidget import LEDIndicatorWidget


class WidgetLaserInformation(QWidget):
    def __init__(self):
        super().__init__()
        self.layout = QGridLayout()

        self.init_UI()



    def init_UI(self):
        rid_group_box = QGroupBox("Capturing Information")
        layout = QGridLayout()



        # Connection state
        self.lbl_conn_state = QLabel("Not connected")
        self.led_conn_state = LEDIndicatorWidget(color="red")
        layout.addWidget(self.led_conn_state, 0, 0)
        layout.addWidget(self.lbl_conn_state, 0, 1)

        # Flag i s moving
        self.lbl_laser_is_moving = QLabel("Laser not moving")
        self.led_laser_is_moving = LEDIndicatorWidget(color="red")
        layout.addWidget(self.led_laser_is_moving, 1, 0)
        layout.addWidget(self.lbl_laser_is_moving, 1, 1)

        # Device state
        #self.lbl_device_state = QLabel("No faults (not connected)")
        #self.led_device_state = LEDIndicatorWidget(color="gray")
        #layout.addWidget(self.led_device_state, 2, 0)
        #layout.addWidget(self.lbl_device_state, 2, 1)

        # Connection state
        self.lbl_capt_device = QLabel("No Capturing device connected")
        self.led_capt_device = LEDIndicatorWidget(color="red")
        layout.addWidget(self.led_capt_device, 2, 0)
        layout.addWidget(self.lbl_capt_device, 2, 1)


        #grid_group_box.setLayout(layout)
        #self.layout.addWidget(layout)
        self.setLayout(layout)

    def set_connection_state(self, state: bool, text: str = None):
        if state:
            self.led_conn_state.set_color("green")
            if text is not None:
                self.lbl_conn_state.setText(text)
            else:
                self.lbl_conn_state.setText("Connected")
        else:
            self.led_conn_state.set_color("red")
            if text is not None:
                self.lbl_conn_state.setText(text)
            else:
                self.lbl_conn_state.setText("Not connected")

    def set_movement_state(self, state: bool, text: str = None):
        if state:
            self.led_laser_is_moving.set_color("green")
            if text is not None:
                self.lbl_laser_is_moving.setText(text)
            else:
                self.lbl_laser_is_moving.setText("Laser moving")
        else:
            self.led_laser_is_moving.set_color("red")
            if text is not None:
                self.lbl_laser_is_moving.setText(text)
            else:
                self.lbl_laser_is_moving.setText("Laser not moving")


    def set_capt_dev_state(self, state: bool, text: str = None):
        if state:
            self.led_capt_device.set_color("green")
            if text is not None:
                self.lbl_capt_device.setText(text)
            else:
                self.lbl_capt_device.setText("Laser moving")
        else:
            self.led_capt_device.set_color("red")
            if text is not None:
                self.lbl_capt_device.setText(text)
            else:
                self.lbl_capt_device.setText("Laser not moving")


from PySide6.QtCore import QObject, Signal


class LaserControlSignals(QObject):
    def __init__(self, parent: QObject = None):
        super().__init__(parent)


    laser_config_changed = Signal()

    # Device status
    connected_changed = Signal(bool)
    port_changed = Signal(str)

    # Device parameter
    laser_moving_to_wavelength_changed = Signal(float)

    current_wavelength_changed = Signal(float)
    min_laser_wavelength_changed = Signal(float)
    max_laser_wavelength_changed = Signal(float)

    deceleration_changed = Signal(float)
    min_deceleration_changed = Signal(float)
    max_deceleration_changed = Signal(float)

    acceleration_changed = Signal(float)
    min_acceleration_changed = Signal(float)
    max_acceleration_changed = Signal(float)


    velocity_changed = Signal(float)
    min_velocity_changed = Signal(float)
    max_velocity_changed = Signal(float)


    laser_is_moving_changed = Signal(bool, float)
    wavelength_sweep_running_changed = Signal(bool)
    laser_at_position_changed = Signal(bool)
    laser_ready_for_sweep_changed = Signal(bool)

    # Device sweep parameter
    sweep_start_wavelength_changed = Signal(float)
    sweep_stop_wavelength_changed = Signal(float)

    connected = Signal(bool)

    capturing_device_changed = Signal(bool)
    capturing_device_connected_changed = Signal(object)

    # current wavelength, min wavelength, max wavelength
    # velocity, acceleration, deceleration
    current_speed_settings = Signal(float, float, float)
    wavelength_sweep_finished = Signal(bool)
    error = Signal(str)
    in_wavelength_sweep_start = Signal(bool)



import ctypes
import os
import pathlib
import shutil
import sys

from WidgetCollection.Tools.PyProjectExtractor import extract_pyproject_info

from . import Helpers

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))



# ======================================================================================================================
# The pyconfig.toml file is needed, to get the metadata. Depending on the installation method (pip or git) the file
# is found in different places.
# ======================================================================================================================
pytoml = Helpers.get_pyprojecttoml()
def try_and_set(func, *args, **kwargs):
    try:
        return func(*args, **kwargs)
    except Exception as e:
        print(f"Error reading '{args[1]}' from {pathlib.Path(args[0])}: {e}")
        return "unknown"

__rootdir__ = os.path.dirname(os.path.realpath(__file__))
__version__ = try_and_set(extract_pyproject_info, pytoml.parent, "version")
__author__ = try_and_set(extract_pyproject_info, pytoml.parent, "author")
__description__ = try_and_set(extract_pyproject_info, pytoml.parent, "description")
__license__ = try_and_set(extract_pyproject_info, pytoml.parent, "license")
__url__ = try_and_set(extract_pyproject_info, pytoml.parent, "url")

# For correctly display the icon in the taskbar
myappid = f'agentsmith29.SacherECLControl.{__version__}'  # arbitrary string
ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

from .controller.LaserDeviceControl import MPLaserDeviceControl as Controller
from .model.LaserControlModel import LaserControlModel as Model
from .view.LaserControlView import LaserControlView as View
from .LaserConfig import LaserConfig as Config



# import mcpy
import ADScopeControl as captdev

from SacherECLControl.LaserConfig import LaserConfig
from SacherECLControl.model.LaserControlSignals import LaserControlSignals


# from SacherECLControl.model.LaserControlSignals import LaserControllerSignals
# from LaserControlOld.LaserConfig import LaserConfig


class LaserControlModel(object):

    def __init__(self, laser_config: LaserConfig):

        self.signals = LaserControlSignals()
        self._laser_config: LaserConfig = laser_config

        self._connected = False
        self._port = None

        self._laser_connected = False
        self._laser_at_position = False
        self._laser_is_moving = False
        self._wavelength_sweep_running = False
        self._laser_ready_for_sweep = False
        # A class which can be used to export the laser Settings

        self._laser_moving_to_wavelength: float = 0
        self._current_wavelength = -1
        self._min_laser_wavelength = 1
        self._max_laser_wavelength = 99999

        self._min_deceleration = -10
        self._max_deceleration = 10

        self._min_acceleration = -10
        self._max_acceleration = 10

        self._min_velocity = -10
        self._max_velocity = 10

        # Capturing device settings
        self._capturing_device_connected: bool = False
        self._capturing_device: captdev.Controller = None


    @property
    def laser_config(self) -> LaserConfig:
        return self._laser_config

    @laser_config.setter
    def laser_config(self, value: LaserConfig):
        self._laser_config = value
        # self.signals.laser_config_changed.emit(self.laser_properties)

    # @property
    # def laser_properties(self) -> LaserProperties:
    #    return LaserProperties(
    #            mcpy.Rectangular(self.acceleration, 0.01, unit='nm/s', k=2),
    #            mcpy.Rectangular(self.deceleration, 0.01, unit='nm/s', k=2),
    #            mcpy.Rectangular(self.velocity, 0.01, unit='nm/s^2', k=2),
    #            (mcpy.Rectangular(self.sweep_start_wavelength, 0.01, unit='nm', k=2), mcpy.Rectangular(self.sweep_stop_wavelength, 0.01, unit='nm', k=2))
    #        )

    @property
    def laser_ready_for_sweep(self):
        return self._laser_ready_for_sweep

    @laser_ready_for_sweep.setter
    def laser_ready_for_sweep(self, value):
        self._laser_ready_for_sweep = value
        self.signals.laser_ready_for_sweep_changed.emit(self._laser_ready_for_sweep)

    @property
    def laser_at_position(self):
        return self._laser_at_position

    @laser_at_position.setter
    def laser_at_position(self, value):
        self._laser_at_position = value
        self.signals.laser_at_position_changed.emit(self.laser_at_position)

    @property
    def sweep_start_wavelength(self):
        return self.laser_config.wl_sweep_start.get()

    @sweep_start_wavelength.setter
    def sweep_start_wavelength(self, value):
        #if value < self.min_laser_wavelength:
        #    raise Exception(f"Sweep start wavelength ({value}) can not be smaller than "
        #                    f"minimum laser wavelength {self.min_laser_wavelength})!")
        #if value >= self.max_laser_wavelength:
        #    raise Exception(f"Sweep start wavelength ({value}) can not be greater or equal to the "
        #                    f"maximum laser wavelength ({self.min_laser_wavelength})!")
        #if value > self.sweep_stop_wavelength:
        #    raise Exception(f"Sweep start wavelength ({value}) can not be greater than "
        #                    f"sweep stop wavelength ({self.sweep_stop_wavelength})!")
        self.laser_config.wl_sweep_start.set(value)
        self.signals.sweep_start_wavelength_changed.emit(self.sweep_start_wavelength)

    @property
    def sweep_stop_wavelength(self) -> float:
        return self.laser_config.wl_sweep_stop.get()

    @sweep_stop_wavelength.setter
    def sweep_stop_wavelength(self, value):
        #if value < self.min_laser_wavelength:
        #    raise Exception(f"Sweep stop wavelength ({value}) can not be smaller than "
        #                    f"minimum laser wavelength {self.min_laser_wavelength})!")
        #if value > self.max_laser_wavelength:
        #    raise Exception(f"Sweep stop wavelength ({value}) can not be greater or equal to the "
        #                    f"maximum laser wavelength ({self.max_laser_wavelength})!")
        #if value < self.sweep_start_wavelength:
        #    raise Exception(f"Sweep stop wavelength ({value}) can not be smaller than "
        #                    f"sweep start wavelength ({self.sweep_start_wavelength})!")
        self.laser_config.wl_sweep_stop.set(value)
        self.signals.sweep_stop_wavelength_changed.emit(self.sweep_stop_wavelength)

    @property
    def connected(self):
        return self._connected

    @connected.setter
    def connected(self, value):
        self._connected = value
        self.signals.connected_changed.emit(self.connected)

    @property
    def port(self):
        return self.laser_config.port.get()

    @port.setter
    def port(self, value):
        self.laser_config.port.set(value)
        self.signals.port_changed.emit(self.port)

    @property
    def laser_is_moving(self):
        return self._laser_is_moving

    @laser_is_moving.setter
    def laser_is_moving(self, value: tuple):
        self._laser_is_moving = value[0]
        self.laser_moving_to_wavelength = value[1]
        self.signals.laser_is_moving_changed.emit(self.laser_is_moving, self.laser_moving_to_wavelength)

    @property
    def laser_moving_to_wavelength(self):
        return self._laser_moving_to_wavelength

    @laser_moving_to_wavelength.setter
    def laser_moving_to_wavelength(self, value):
        self._laser_moving_to_wavelength = value
        self.signals.laser_moving_to_wavelength_changed.emit(self.laser_moving_to_wavelength)

    @property
    def current_wavelength(self):
        return self._current_wavelength

    @current_wavelength.setter
    def current_wavelength(self, value):
        self._current_wavelength = value
        self.signals.current_wavelength_changed.emit(self.current_wavelength)

    @property
    def min_laser_wavelength(self):
        return self._min_laser_wavelength

    @min_laser_wavelength.setter
    def min_laser_wavelength(self, value):
        self._min_laser_wavelength = value
        self.signals.min_laser_wavelength_changed.emit(self.min_laser_wavelength)

    @property
    def max_laser_wavelength(self):
        return self._max_laser_wavelength

    @max_laser_wavelength.setter
    def max_laser_wavelength(self, value):
        self._max_laser_wavelength = value
        self.signals.max_laser_wavelength_changed.emit(self.max_laser_wavelength)

    # ==================================================================================================================
    @property
    def deceleration(self):
        return self.laser_config.deceleration.get()

    @deceleration.setter
    def deceleration(self, value):
        self.laser_config.deceleration.set(value)
        self.signals.deceleration_changed.emit(self.deceleration)

    @property
    def min_deceleration(self):
        return self._min_deceleration

    @min_deceleration.setter
    def min_deceleration(self, value):
        self._min_acceleration = value
        self.signals.min_deceleration_changed.emit(self.min_deceleration)

    @property
    def max_deceleration(self):
        return self._max_deceleration

    @max_deceleration.setter
    def max_deceleration(self, value):
        self._max_deceleration = value
        self.signals.max_deceleration_changed.emit(self.max_deceleration)

    # ==================================================================================================================
    @property
    def acceleration(self):
        return self.laser_config.acceleration.get()

    @acceleration.setter
    def acceleration(self, value):
        self.laser_config.acceleration.set(value)
        self.signals.acceleration_changed.emit(self.acceleration)

    @property
    def min_acceleration(self):
        return self._min_acceleration

    @min_acceleration.setter
    def min_acceleration(self, value):
        self._min_acceleration = value
        self.signals.min_acceleration_changed.emit(self.min_acceleration)

    @property
    def max_acceleration(self):
        return self._max_acceleration

    @max_acceleration.setter
    def max_acceleration(self, value):
        self._max_acceleration = value
        self.signals.max_acceleration_changed.emit(self.max_acceleration)

    # ==================================================================================================================

    @property
    def velocity(self):
        return self.laser_config.velocity.get()

    @velocity.setter
    def velocity(self, value):
        self.laser_config.velocity.set(value)
        self.signals.velocity_changed.emit(self.velocity)

    @property
    def min_velocity(self):
        return self._min_acceleration

    @min_velocity.setter
    def min_velocity(self, value):
        self._min_velocity = value
        self.signals.min_velocity_changed.emit(self._min_velocity)

    @property
    def max_velocity(self):
        return self._max_acceleration

    @max_velocity.setter
    def max_velocity(self, value):
        self._max_velocity = value
        self.signals.max_velocity_changed.emit(self._max_velocity)

    # ==================================================================================================================

    @property
    def laser_connected(self):
        return self._laser_connected

    @property
    def wavelength_sweep_running(self):
        return self._wavelength_sweep_running

    @wavelength_sweep_running.setter
    def wavelength_sweep_running(self, value):
        self._wavelength_sweep_running = value
        self.signals.wavelength_sweep_running_changed.emit(self._wavelength_sweep_running)

    # ==================================================================================================================
    @property
    def capturing_device_connected(self):
        return self._capturing_device_connected

    @capturing_device_connected.setter
    def capturing_device_connected(self, value):
        self._capturing_device_connected = value
        self.signals.capturing_device_connected_changed.emit(self.capturing_device_connected)

    @property
    def capturing_device(self) -> captdev.Controller:
        return self._capturing_device

    @capturing_device.setter
    def capturing_device(self, value: captdev.Controller):
        self._capturing_device = value
        self.signals.capturing_device_changed.emit(self._capturing_device)


import logging
import sys

from PySide6.QtWidgets import QApplication

import Laser
from ConfigHandler.controller.VAutomatorConfig import VAutomatorConfig
from generics.logger import setup_logging

if __name__ == "__main__":
    app = QApplication()
    setup_logging()

    logging.warning("[TEST] LaserControlWindow.py is not meant to be run as a script.")

    vaut_config = VAutomatorConfig.load_config("../configs/init_config.yaml")

    laser_model = Laser.Model()
    laser_controller = Laser.Controller(laser_model, vaut_config.laser_config)
    laser_window = Laser.ControlWindow(laser_model, laser_controller)
    # test_win = StructureSelector()

    laser_window.show()
    sys.exit(app.exec())


import logging
import os
from pathlib import Path

import confPy6 as cfg


class LaserConfig(cfg.ConfigNode):

    def __init__(self) -> None:
        super().__init__()

        # Set the path to the EposCmd64.dll
        self.epos_dll = cfg.Field(Path("./EposCmd64.dll"))
        # Set the path to the SacherMotorControl.pyd
        self.motor_control_pyd = cfg.Field(
            Path(f"./SacherMotorControl.pyd"),
            env_var="MOTOR_CONTROL_PYD")

        self.wl_sweep_start = cfg.Field(857)
        self.wl_sweep_stop = cfg.Field(870)
        self.velocity = cfg.Field(2.0)
        self.acceleration = cfg.Field(1.0)
        self.deceleration = cfg.Field(1.0)
        self.available_ports = cfg.Field(['USB0', 'USB1', 'USB2', 'USB3', 'USB4', 'USB5', 'USB6', 'USB7', 'USB8', 'USB9'])
        self.port = cfg.Field("USB0")

        self.register()



import logging
import pathlib
import time
from random import randint

from SacherECLControl.libs.LaserSceleton import LaserScelton
import confPy6 as conf


class LaserLibSimulatorConfig(conf.ConfigNode):

    def __init__(self) -> None:
        super().__init__()
        self.currentWavelengthPositionIs: conf.Field[float] = conf.Field(0.0,
                                                                          friendly_name="Current wavelength position",
                                                                          description="Current wavelength position in nm")

        self.current_wavelength: conf.Field[float] = conf.Field(857.0, friendly_name="Current wavelength",
                                                                 description="Current wavelength in nm")

        self.min_wavelength: conf.Field[float] = conf.Field(830.0, friendly_name="Min wavelength",
                                                                description="Minimum wavelength in nm")

        self.max_wavelength: conf.Field[float] = conf.Field(880.0, friendly_name="Max wavelength",
                                                                description="Maximum wavelength in nm")

        self.velocity: conf.Field[float] = conf.Field(1.0, friendly_name="Velocity",
                                                       description="Velocity in nm/s")
        self.acceleration: conf.Field[float] = conf.Field(2.0, friendly_name="Acceleration",
                                                           description="Acceleration in nm/s^2")
        self.deceleration: conf.Field[float] = conf.Field(2.0, friendly_name="Deceleration",
                                                           description="Deceleration in nm/s^2")

        self.register()


# Implements the Simulator for Version
class Motor(LaserScelton):
    connected = False

    def __init__(self, logger=None) -> None:
        super().__init__()
        self.pref = "Laser (Simulator)"
        self.port_list = ['Simulator']

        self._connected = False

        pconfath = ".sim/laser_simulator.yaml"
        self.conf = LaserLibSimulatorConfig()
        self.conf.autosave(enable=True, path=pconfath)

        # check if .sim/laser_simulator.yaml exists
        if pathlib.Path(pconfath).exists():
            self.conf.load(pconfath, True)

        self.conf.module_log_enabled = True
        self.conf.module_log_level = logging.DEBUG

        self.port = "SIM"
        # self.connected = False

        self.logger = logger or logging.getLogger(f"{__name__} (Sim)")
        self.logger.info(f"{self.pref}: Laser Simulator initialized.")

    # ==================================================================================================================
    @property
    def currentWaveLengthPositionIs(self) -> float:
        return self.conf.currentWavelengthPositionIs.get()

    @currentWaveLengthPositionIs.setter
    def currentWaveLengthPositionIs(self, value: float) -> None:
        self.conf.currentWavelengthPositionIs.set(value)

    @property
    def currentWavelength(self) -> float:
        return self.conf.current_wavelength.get()

    @currentWavelength.setter
    def currentWavelength(self, value: float) -> None:
        self.conf.current_wavelength.set(value)

    @property
    def velocity(self) -> float:
        return self.conf.velocity.get()

    @velocity.setter
    def velocity(self, value: float) -> None:
        self.conf.velocity.set(value)

    @property
    def acceleration(self) -> float:
        return self.conf.acceleration.get()

    @acceleration.setter
    def acceleration(self, value: float) -> None:
        self.conf.acceleration.set(value)

    @property
    def deceleration(self) -> float:
        return self.conf.deceleration.get()

    @deceleration.setter
    def deceleration(self, value: float) -> None:
        self.conf.deceleration.set(value)

    # ==================================================================================================================

    # ==================================================================================================================
    # Connection functions
    # ==================================================================================================================
    def connect(self, port: str = "USB0") -> None:
        """
        Establishes a connection to the Epos and reads all the needed data saved on the internal
        registry.
        :param port: usb port on which the connection should be established, default=’USB0’
        :return: None
        """
        if self.connected:
            raise Exception("Already connected to Laser Simulator")
        self.logger.debug(f"{self.pref}: Connecting to Laser Simulator.")
        time.sleep(randint(1, 2))
        self._connected = True
        self.logger.info(f"{self.pref}: Connected to Laser Simulator.")

    def disconnect(self) -> None:
        """
        Closes the connection to the Epos.
        :return:
        """
        self.logger.info(f"{self.pref}: Disconnecting from Laser Simulator")
        self._connected = False

    # ==================================================================================================================
    # Laser information/status functions
    # ==================================================================================================================

    def getWavelength(self) -> float:
        """
        Returns the wavelength from the current motor position stored on the epos.
        :return: Wavelength in nm (float)
        """
        self.logger.debug(f"{self.pref}: Current wavelength is {self.currentWavelength} nm")
        return self.currentWavelength

    def getWavelengthMinMax(self) -> tuple:
        """
        Return the minimum and maximum wavelength of the system.
        :return: tuple(float,float) minimum wavelength, maximum wavelength
        """
        self.logger.debug(f"{self.pref}: Min/Max wavelength "
                          f"{self.conf.min_wavelength.get()}/{self.conf.min_wavelength.get()} nm")
        return self.conf.min_wavelength.get(), self.conf.max_wavelength.get()

    def setVelocityParameter(self, velocity: float, acceleration: float, deceleration) -> None:
        """
         Sets the velocity parameters on the Epos.
        :param velocity:  velocity of the system in nm/s
        :param acceleration: acceleration of the system in nm/s2
        :param deceleration: deceleration of the system in nm/s2
        :return: None
        """

        self.logger.info(f"{self.pref}:  set to {acceleration} nm/s^2")
        self.logger.info(f"{self.pref}: Deceleration set to {deceleration} nm/s^2")
        self.velocity = velocity
        self.acceleration = acceleration
        self.deceleration = deceleration
        self.logger.info(f"{self.pref}: Velocity/acceleration/deceleration set to "
                         f"{velocity}(nm/s)/{acceleration}(nm/s^2)/{deceleration}(nm/s^2)")

    def getVelocityParameter(self) -> tuple:
        """
        Returns the velocity in nm/s, acceleration and deceleration in nm/s2 stored on the
        :return: tuple(float, float, float) velocity, acceleration, deceleration
        """
        self.logger.info(f"{self.pref}: Current velocity/acceleration/deceleration set to "
                         f"{self.velocity}(nm/s)/{self.acceleration}(nm/s^2)/{self.deceleration}(nm/s^2)")
        return (self.velocity, self.acceleration, self.deceleration)

    def getMoving(self) -> bool:
        """
        Returns true or false whether the motor is moving or not.
        :return: True, if moving, false otherwise
        """

    # =================================================================================================================
    # Movement functions
    # =================================================================================================================
    def moveToWavelength(self, wavelength: float, highPrecision: bool, trigger: int) -> None:
        """
        Moves the Motor to a given Wavelength.
        :param wavelength: The wavelength to move to given in nm
        :param highPrecision: High Precision Mode activated. When the relative movement is negative the Motor
        drives 10000 steps below the goal to always come up to it with a positive movement.
        If using a ballscrew system the steps will be reduced to 3000.
        :param trigger:
            0 - no trigger activated
            1 - Position Compare trigger activated
            2 - Motion trigger activated
        :return: None
        """

        num_wavelength = int(wavelength)

        #def calcMovementTime():
        #    et_acc_dec = self.velocity / self.acceleration
        #    wl_after_acc = self.currentWavelength + 0.5 * self.acceleration * et_acc_dec ** 2
        #    wl_before_dec = num_wavelength - 0.5 * self.deceleration * et_acc_dec ** 2
        #    eta = 2 * et_acc_dec + (wl_before_dec - wl_after_acc) / self.velocity
        #    # laser need eta in ms
        #    steps_per_second = (eta / 100)*1000
        #    return steps_per_second

        print(f"Moving from {self.currentWavelength} to {num_wavelength}")
        if int(self.currentWavelength) > num_wavelength:
            print(f"Going down: {self.currentWavelength} -> {num_wavelength}")
            for i in range(int(self.currentWavelength)*10, num_wavelength*10, -int(self.velocity)):
                print(f"{i/10} |  ", end="")
                time.sleep(0.1)
        else:
            print(f"Going up: {self.currentWavelength} -> {num_wavelength}")
            for i in range(int(self.currentWavelength)*10, num_wavelength*10, int(self.velocity)):
                print(f"{i/10} |  ", end="")
                time.sleep(0.1)
        self.currentWavelength = num_wavelength
        print(f"Done: {self.currentWavelength}")

    def moveSteps(self, steps: int, trigger: int) -> None:
        """
        Moves A certain amount of steps up to 10000 in either direction.
        Parameter:
        :param steps: Number of steps to move (int)
        :param trigger: (int)
                    0 - no trigger activated
                    1 - Position Compare trigger activated
                    2 - Motion trigger activated
        :return: None
        """
        self.logger.info(f"{self.pref}: Moving {steps} steps")

    def stopMovement(self) -> None:
        """
        Stops the movement of the motor.
        :return: None
        """
        self.logger.info(f"{self.pref}: Stopping movement")

    # ==================================================================================================================
    # Calibration functions#
    # ==================================================================================================================
    def calibrate(self, wavelength: float) -> None:
        """
        Sets the current motor position to the wavelength position according to the given wavelength.
        :param wavelength (float): wavelength in nm to calibrate the system
        :return: None
        """
        self.logger.info(f"{self.pref}: Calibrating to {wavelength} nm")

    # ==================================================================================================================
    # Trigger Methods
    # ==================================================================================================================

    def triggerParameter(self, intervalWidth) -> None:
        """
        Sets the parameters for the position compare trigger
        :param intervalWidth: Interval width for the trigger signal in nm
        :return: None
        """
        self.logger.info(f"{self.pref}: Setting trigger parameter to {intervalWidth} nm")

    def reverseMotionTrigger(self) -> None:
        """
        Reverses the digital output for MotionTrigger. To change between active while moving
        or active while standing.
        """
        self.logger.info(f"{self.pref}: Reversing Motion Trigger")




import logging
import os
import pathlib
import sys
from PySide6.QtWidgets import QApplication
from rich.logging import RichHandler

sys.path.append('../src')
import SacherECLControl as Laser

if __name__ == "__main__":
    FORMAT = "%(name)s %(message)s"
    logging.basicConfig(
        level="DEBUG", format=FORMAT, datefmt="[%X]", handlers=[
            RichHandler(rich_tracebacks=True)
        ]
    )
    if not QApplication.instance():
        app = QApplication(sys.argv)
    else:
        app = QApplication.instance()

    conf = Laser.Config()
    conf.sav()
    #conf.load('./LaserConfig.yaml', as_auto_save=True)
    conf.autosave(True, './LaserConfig.yaml')

    # Set the path to the EposCmd64.dll and the SacherMotorControl.pyd
    conf.epos_dll.set(pathlib.Path(
        f'{Laser.__rootdir__}/libs/SacherLib/PythonMotorControlClass/EposCmd64.dll'))

    conf.motor_control_pyd.set(pathlib.Path(
        f'{Laser.__rootdir__}/libs/SacherLib/PythonMotorControlClass/'
        f'lib/Python312/SacherMotorControl.pyd'))



    conf.module_log_level = logging.DEBUG
    conf.module_log_enabled = True

    model = Laser.Model(conf)
    controller = Laser.Controller(model, None)
    controller.internal_log_level = logging.DEBUG
    controller.internal_log_enabled = True
    window = Laser.View(model, controller)

    #controller.start_wavelength_sweep.emit(
    #    model.sweep_start_wavelength,
    #    model.sweep_stop_wavelength,
    #)

    window.show()

    sys.exit(app.exec())



import os
import pathlib
import sys
import time
from multiprocessing import Value

import mpPy6
from mpPy6.CProperty import CProperty

from SacherECLControl import Helpers, __rootdir__

# Copy the dll to the startup directory

try:
    spath = str(pathlib.Path(os.getenv("MOTOR_CONTROL_PYD")).absolute().parent.as_posix())
    sys.path.append(spath)
    import SacherMotorControl as LaserLib

except Exception as e:
    print(f"Warning: Could not import SacherMotorControl: {e}. Using LaserLibSimulator instead.")
    from SacherECLControl.libs import LaserLibSimulator as LaserLib


#from SacherECLControl.libs import LaserLibSimulator as LaserLib


class MPLaserDevice(mpPy6.CProcess):

    def __init__(self, state_queue, cmd_queue,
                 laser_moving_flag: Value,
                 laser_finished_flag: Value,
                 start_capture_flag: Value,
                 dll_copy_path,
                 kill_flag: Value,
                 internal_log, internal_log_level, log_file):
        super().__init__(state_queue, cmd_queue,
                         kill_flag=kill_flag,
                         internal_log=internal_log, internal_log_level=internal_log_level, log_file=log_file)


        self.dll_copy_path = dll_copy_path
        #import importlib.util
        #import sys
        #spec = importlib.util.spec_from_file_location(module_name, module_name_path)
        #foo = importlib.util.module_from_spec(spec)
        #sys.modules[module_name] = foo
        #spec.loader.exec_module(foo)

        # if not self.logger.handlers:
        #     self.logger.setLevel(level=logging.DEBUG)
        # self.logger.disabled = False
        # print(self.logger)
        # self.logger.info(f"Created logger for {self.__class__.__name__}({os.getpid()})")
        self._connected = False
        self._current_wavelength = -1

        self._min_wavelength = 0
        self._max_wavelength = 0
        self._velocity = 0
        self._acceleration = 0
        self._deceleration = 0

        self._laser_moving = False

        self.laser_moving_flag = laser_moving_flag
        self.laser_finished_flag = laser_finished_flag
        self.start_capture_flag = start_capture_flag

        self._wavelength_sweep_running = False

    def postrun_init(self):
        try:
            self.logger.info(f"Copying DLL to {self.dll_copy_path}")
            Helpers.copyEposDLL(f"{self.dll_copy_path}", logger=self.logger)
        except Exception as e:
            self.logger.error(f"Error copying DLL: {e}")
        self.laser = LaserLib.Motor()
    # ==================================================================================================================
    # Properties
    # ==================================================================================================================
    @CProperty
    def connected(self):
        """ Returns if the laser is connected. """
        return self._connected

    @connected.setter('connected_changed')
    def connected(self, value: float):
        """ Sets the connected state of the laser. Only used internally by the process. """
        self._connected = value

    @CProperty
    def current_wavelength(self):
        """
        Returns the wavelength from the current motor position stored on the epos
        """
        return self._current_wavelength

    @current_wavelength.setter('current_wavelength_changed')
    def current_wavelength(self, value: float):
        """
        Sets the current wavelength of the laser. Only used internally by the process.
        :param value: Wavelength read by the laser library
        """
        self._current_wavelength = value

    @CProperty
    def min_wavelength(self):
        """
        :return: Returns the minimum wavelength the laser can be set to.
        """
        return self._min_wavelength

    @min_wavelength.setter('min_wavelength_changed')
    def min_wavelength(self, value: float):
        """
        Sets the minimum wavelength of the laser. Only used internally by the process.
        :param value: The minimum wavelength read by the laser library
        :return:
        """
        self._min_wavelength = value

    @CProperty
    def max_wavelength(self):
        """
        :return: Returns the maximum wavelength the laser can be set to.
        """
        return self._max_wavelength

    @max_wavelength.setter('max_wavelength_changed')
    def max_wavelength(self, value: float):
        """ Sets the maximum wavelength of the laser. Only used internally by the process."""
        self._max_wavelength = value

    @CProperty
    def velocity(self):
        """
        :return: Returns the velocity of the laser.
        :return:
        """
        return self._velocity

    @velocity.setter('velocity_changed')
    def velocity(self, value: float):
        """ Sets the velocity of the laser. Only used internally by the process."""
        self._velocity = value

    @CProperty
    def acceleration(self):
        """
        :return: Returns the acceleration of the laser.
        :return:
        """
        return self._acceleration

    @acceleration.setter('acceleration_changed')
    def acceleration(self, value: float):
        """
        Sets the acceleration of the laser. Only used internally by the process.
        :param value:
        :return:
        """
        self._acceleration = value

    # ==================================================================================================================

    @CProperty
    def deceleration(self):
        """
        :return: Returns the deceleration of the laser.
        :return:
        """
        return self._deceleration

    @deceleration.setter('deceleration_changed')
    def deceleration(self, value: float):
        """
        Sets the deceleration of the laser. Only used internally by the process.
        :param value:
        :return:
        """
        self._deceleration = value

    # ==================================================================================================================
    # Connection
    # ==================================================================================================================
    @mpPy6.CProcess.register_signal()
    def connect_device(self, usb_port: str) -> bool:
        """
        Connect to the laser on the specified USB port.
        :param usb_port: USB port to connect to
        :return: True if connected, False otherwise
        """
        self.logger.info(f"Connecting to laser on port {usb_port}")

        try:
            self.laser.connect(usb_port)
            self.connected = True
        except Exception as e:
            self.logger.error(f"Error connecting to laser: {e}")
            self.connected = False
            raise e

        self.get_laser_settings()
        return self._connected

    @mpPy6.CProcess.register_signal()
    def disconnect_device(self) -> bool:
        """
        Closes the connection to the Epos.
        """
        self.logger.info(f"Disconnecting from laser.")

        try:
            self.laser.disconnect()
            self.connected = False
        except Exception as e:
            self.logger.error(f"Error disconnecting from laser: {e}")
            self.connected = False
            raise e

        self.get_laser_settings()
        return self._connected

    # ==================================================================================================================#
    #
    # ==================================================================================================================#
    @CProperty
    def wavelength_sweep_running(self):
        return self._wavelength_sweep_running

    @wavelength_sweep_running.setter('wavelength_sweep_running_changed')
    def wavelength_sweep_running(self, value: bool):
        self._wavelength_sweep_running = value

    # ==================================================================================================================

    # ==================================================================================================================

    def _check_laser_state(self) -> bool:
        """
        Check if the laser is connected and the laser object is initialized.
        This is a helper function to avoid code duplication.
        :param func:
        :return:
        """
        if self.laser is None or self._connected is False:
            return False
        else:
            return True

    # ==================================================================================================================

    # ==================================================================================================================
    # Reading functions. Updates the laser settings of this class.
    # ==================================================================================================================
    def get_current_wavelength(self) -> float:
        if self.laser is None or self._connected is False:
            self.current_wavelength = 0
            return -1
        else:
            self.current_wavelength = float(self.laser.getWavelength())
        return self._current_wavelength

    def get_min_wavelength(self) -> float:
        if self.laser is None or self._connected is False:
            self.min_wavelength = 0
            return -1
        else:
            self.min_wavelength = float(self.laser.getWavelengthMinMax()[0])

        return self._min_wavelength

    def get_max_wavelength(self) -> float:
        if self.laser is None or self._connected is False:
            self.max_wavelength = 0
            return -1
        else:
            self.max_wavelength = float(self.laser.getWavelengthMinMax()[1])
        return self._max_wavelength

    def get_velocity(self) -> float:
        if self.laser is None or self._connected is False:
            self.velocity = 0
            return -1
        else:
            self.velocity = float(self.laser.getVelocityParameter()[0])
        return self._velocity

    @mpPy6.CProcess.register_signal()
    def set_velocity(self, velocity: float) -> None:
        """ Set the velocity of the laser. """
        if self._check_laser_state():
            self.laser.setVelocityParameter(velocity, self._acceleration, self._deceleration)
            self.get_laser_settings()
        else:
            raise Exception("Velocity could not be set. Laser is not connected.")

    def get_acceleration(self) -> float:
        if self.laser is None or self._connected is False:
            self.acceleration = 0
            return -1
        else:
            self.acceleration = float(self.laser.getVelocityParameter()[1])
        return self._acceleration

    @mpPy6.CProcess.register_signal()
    def set_acceleration(self, acceleration: float) -> None:
        """ Set the acceleration of the laser. """
        if self._check_laser_state():
            self.laser.setVelocityParameter(self._velocity, acceleration, self._deceleration)
            self.get_laser_settings()
        else:
            raise Exception("Acceleration could not be set. Laser is not connected.")

    def get_deceleration(self) -> float:
        if self.laser is None or self._connected is False:
            self.deceleration = 0
            return -1
        else:
            self.deceleration = float(self.laser.getVelocityParameter()[2])
        return self._deceleration

    @mpPy6.CProcess.register_signal()
    def set_deceleration(self, deceleration: float) -> None:
        """ Set the deceleration of the laser. """
        if self._check_laser_state():
            self.laser.setVelocityParameter(self._velocity, self._acceleration, deceleration)
            self.get_laser_settings()
        else:
            raise Exception("Deceleration could not be set. Laser is not connected.")

    @mpPy6.CProcess.register_signal(signal_name="laser_settings_changed")
    def get_laser_settings(self, *args, **kwargs):
        self.logger.info(f"Reading laser settings.")

        self.get_min_wavelength()
        self.get_max_wavelength()
        self.get_current_wavelength()
        self.get_velocity()
        self.get_acceleration()
        self.get_deceleration()

    @CProperty
    def laser_is_moving(self):
        return self._laser_moving

    @laser_is_moving.setter('laser_is_moving_changed')
    def laser_is_moving(self, value: tuple):
        self.laser_moving_flag.value = int(value[0])
        self._laser_moving = value

    @mpPy6.CProcess.register_signal(postfix="_changed")
    def movement_finished(self, finished: bool):
        self.laser_finished_flag.value = finished
        return self.laser_finished_flag.value

    @mpPy6.CProcess.register_signal(postfix="_finished")
    def move_to_wavelength(self, usb_port: str = None,
                           wavelength: float = None, capture: bool = False, *args, **kwargs):

        if self.laser is None or self._connected is False:
            return -1
        else:

            self.get_laser_settings()
            self.laser_is_moving = (False, wavelength)
            self.laser_finished_flag.value = False
            self.logger.info(f"**** Go to selected wavelength. Started moving laser to {wavelength}. ****")
            time_start = time.time()
            self.laser_is_moving = (True, wavelength)
            if capture:
                self.start_capture_flag.value = int(True)
                self.logger.info(
                    f"******************** Capture flag set to {self.start_capture_flag.value} **********************")

            #time.sleep(1)
            self.laser.moveToWavelength(wavelength, False, trigger=0)
            self.start_capture_flag.value = int(False)
            self._module_logger.info(
                f"******************** Capture flag set to {self.start_capture_flag.value} **********************")
            self.laser_is_moving = (False, wavelength)
            time_end = time.time()
            # if capture_device_started_flag is not None:
            #    capture_device_started_flag.value = False
            # laser_finished_flag.value = True
            # laser_moving_flag.value = False
            self.logger.info(f"Go to selected wavelength finished.")

            # current_wavelength.value = con.current_wavelength
            self.logger.info(
                f">>> Current Wavelength: {self.get_current_wavelength()}. Took {time_end - time_start} seconds to move.")
            self.get_laser_settings()
            # laser_connected_flag.value = False  # We need to manually set this
            return self.get_current_wavelength()

    def wavelength_sweep(self, usb_port: str = None,
                         wavelength_start: float = None,
                         wavelength_end: float = None, *args, **kwargs):

        # laser_finished_flag.value = False
        # laser_state.value.connected = False
        self.wavelength_sweep_running = True
        self.get_laser_settings(usb_port)
        self.logger.info(f"Starting sweep from {wavelength_start} to {wavelength_end}.")
        self.logger.info(f"Resetting laser to start wavelength {wavelength_start}. "
                         f"Current wavelength is {self.current_wavelength}")
        self.move_to_wavelength(usb_port, wavelength_start)
        self.logger.info(f"Starting sweep from {wavelength_start} to {wavelength_end}.")
        self.move_to_wavelength(usb_port, wavelength_end, capture=True)
        self.logger.info(f"Finished sweep from {wavelength_start} to {wavelength_end}.")
        self.wavelength_sweep_running = False
        # laser_state.connected = False



import logging
import sys
import os
import pathlib
from multiprocessing import Value

from rich.logging import RichHandler
from PySide6.QtWidgets import QApplication

import ADScopeControl as captdev

file_path, _ = os.path.split(os.path.realpath(__file__))
src_path = f"{file_path}/../src"
sys.path.append(src_path)

import SacherECLControl as Laser

if __name__ == "__main__":
    FORMAT = "%(name)s %(message)s"
    logging.basicConfig(
        level="DEBUG", format=FORMAT, datefmt="[%X]", handlers=[
            RichHandler(rich_tracebacks=True)
        ]
    )

    if not QApplication.instance():
        app = QApplication(sys.argv)
    else:
        app = QApplication.instance()

    conf_capt_dev = captdev.Config()
    #conf_capt_dev.load("CaptDeviceConfig.yaml")
    conf_capt_dev.autosave()


    start_capture_flag = Value('i', 0)

    capt_dev_model = captdev.Model(conf_capt_dev)
    capt_dev_controller = captdev.Controller(capt_dev_model, start_capture_flag)
    capt_dev_window = captdev.View(capt_dev_model, capt_dev_controller)


    conf = Laser.Config()
    # Set the path to the EposCmd64.dll and the SacherMotorControl.pyd
    conf.epos_dll.set(pathlib.Path(
        f'{Laser.__rootdir__}/libs/SacherLib/PythonMotorControlClass/EposCmd64.dll'))

    conf.motor_control_pyd.set(pathlib.Path(
        f'{Laser.__rootdir__}/libs/SacherLib/PythonMotorControlClass/'
        f'lib/Python312/SacherMotorControl.pyd'))

    model = Laser.Model(conf)
    controller = Laser.Controller(model, start_capture_flag)
    window = Laser.View(model, controller)
    capt_dev_window.show()

    controller.supervise(capt_dev_controller)

    window.show()


    sys.exit(app.exec())